@@ -196,12 +196,18 @@ class ConfigMain::Impl {
         [](const std::string & value)->std::uint32_t{
             if (value == "<off>")
                 return 0;
+            std::int32_t value_i;
             try {
-                return std::stoul(value);
+                value_i = std::stol(value);
             }
             catch (...) {
-                return 0;
+                throw Option::InvalidValue(tfm::format(_("invalid value")));
             }
+            if (value_i == 1)
+                throw Option::InvalidValue(tfm::format(_("value 1 is not allowed")));
+            if (value_i < 0)
+                throw Option::InvalidValue(tfm::format(_("negative value is not allowed")));
+            return (std::uint32_t)value_i;
         }
     };
 
@@ -279,6 +285,7 @@ class ConfigMain::Impl {
     OptionBool ignorearch{false};
     OptionString module_platform_id{nullptr};
     OptionBool module_stream_switch{false};
+    OptionBool module_obsoletes{false};
 
     OptionString user_agent{getUserAgent()};
     OptionBool countme{false};
@@ -291,6 +298,8 @@ class ConfigMain::Impl {
     OptionBool fastestmirror{false};
     OptionStringList excludepkgs{std::vector<std::string>{}};
     OptionStringList includepkgs{std::vector<std::string>{}};
+    OptionStringList exclude_from_weak{std::vector<std::string>{}};
+    OptionBool exclude_from_weak_autodetect{true};
     OptionString proxy{""};
     OptionString proxy_username{nullptr};
     OptionString proxy_password{nullptr};
@@ -340,14 +349,20 @@ class ConfigMain::Impl {
 
     OptionSeconds timeout{30};
     OptionNumber<std::uint32_t> max_parallel_downloads{3, 1};
+    OptionNumber<std::uint32_t> max_downloads_per_mirror{3, 1};
     OptionSeconds metadata_expire{60 * 60 * 48};
     OptionString sslcacert{""};
     OptionBool sslverify{true};
     OptionString sslclientcert{""};
     OptionString sslclientkey{""};
+    OptionString proxy_sslcacert{""};
+    OptionBool proxy_sslverify{true};
+    OptionString proxy_sslclientcert{""};
+    OptionString proxy_sslclientkey{""};
     OptionBool deltarpm{true};
     OptionNumber<std::uint32_t> deltarpm_percentage{75};
     OptionBool skip_if_unavailable{false};
+    OptionBool sslverifystatus{false};
 };
 
 ConfigMain::Impl::Impl(Config & owner)
@@ -436,6 +451,7 @@ ConfigMain::Impl::Impl(Config & owner)
     owner.optBinds().add("ignorearch", ignorearch);
     owner.optBinds().add("module_platform_id", module_platform_id);
     owner.optBinds().add("module_stream_switch", module_stream_switch);
+    owner.optBinds().add("module_obsoletes", module_obsoletes);
     owner.optBinds().add("user_agent", user_agent);
     owner.optBinds().add("countme", countme);
     owner.optBinds().add("protect_running_kernel", protect_running_kernel);
@@ -462,7 +478,13 @@ ConfigMain::Impl::Impl(Config & owner)
             optionTListAppend(includepkgs, priority, value);
         }, nullptr, true
     );
+    owner.optBinds().add("exclude_from_weak", exclude_from_weak,
+        [&](Option::Priority priority, const std::string & value){
+            optionTListAppend(exclude_from_weak, priority, value);
+        }, nullptr, true
+    );
 
+    owner.optBinds().add("exclude_from_weak_autodetect", exclude_from_weak_autodetect);
     owner.optBinds().add("proxy", proxy);
     owner.optBinds().add("proxy_username", proxy_username);
     owner.optBinds().add("proxy_password", proxy_password);
@@ -486,14 +508,20 @@ ConfigMain::Impl::Impl(Config & owner)
     owner.optBinds().add("throttle", throttle);
     owner.optBinds().add("timeout", timeout);
     owner.optBinds().add("max_parallel_downloads", max_parallel_downloads);
+    owner.optBinds().add("max_downloads_per_mirror", max_downloads_per_mirror);
     owner.optBinds().add("metadata_expire", metadata_expire);
     owner.optBinds().add("sslcacert", sslcacert);
     owner.optBinds().add("sslverify", sslverify);
     owner.optBinds().add("sslclientcert", sslclientcert);
     owner.optBinds().add("sslclientkey", sslclientkey);
+    owner.optBinds().add("proxy_sslcacert", proxy_sslcacert);
+    owner.optBinds().add("proxy_sslverify", proxy_sslverify);
+    owner.optBinds().add("proxy_sslclientcert", proxy_sslclientcert);
+    owner.optBinds().add("proxy_sslclientkey", proxy_sslclientkey);
     owner.optBinds().add("deltarpm", deltarpm);
     owner.optBinds().add("deltarpm_percentage", deltarpm_percentage);
     owner.optBinds().add("skip_if_unavailable", skip_if_unavailable);
+    owner.optBinds().add("sslverifystatus", sslverifystatus);
 }
 
 ConfigMain::ConfigMain() { pImpl = std::unique_ptr<Impl>(new Impl(*this)); }
@@ -572,6 +600,7 @@ OptionBool & ConfigMain::ignorearch() { return pImpl->ignorearch; }
 
 OptionString & ConfigMain::module_platform_id() { return pImpl->module_platform_id; }
 OptionBool & ConfigMain::module_stream_switch() { return pImpl->module_stream_switch; }
+OptionBool & ConfigMain::module_obsoletes() { return pImpl->module_obsoletes; }
 OptionString & ConfigMain::user_agent() { return pImpl->user_agent; }
 OptionBool & ConfigMain::countme() { return pImpl->countme; }
 OptionBool & ConfigMain::protect_running_kernel() {return pImpl->protect_running_kernel; }
@@ -582,6 +611,8 @@ OptionString & ConfigMain::cachedir() { return pImpl->cachedir; }
 OptionBool & ConfigMain::fastestmirror() { return pImpl->fastestmirror; }
 OptionStringList & ConfigMain::excludepkgs() { return pImpl->excludepkgs; }
 OptionStringList & ConfigMain::includepkgs() { return pImpl->includepkgs; }
+OptionStringList & ConfigMain::exclude_from_weak() { return pImpl->exclude_from_weak; }
+OptionBool & ConfigMain::exclude_from_weak_autodetect() { return pImpl->exclude_from_weak_autodetect; }
 OptionString & ConfigMain::proxy() { return pImpl->proxy; }
 OptionString & ConfigMain::proxy_username() { return pImpl->proxy_username; }
 OptionString & ConfigMain::proxy_password() { return pImpl->proxy_password; }
@@ -599,14 +630,20 @@ OptionEnum<std::string> & ConfigMain::ip_resolve() { return pImpl->ip_resolve; }
 OptionNumber<float> & ConfigMain::throttle() { return pImpl->throttle; }
 OptionSeconds & ConfigMain::timeout() { return pImpl->timeout; }
 OptionNumber<std::uint32_t> & ConfigMain::max_parallel_downloads() { return pImpl->max_parallel_downloads; }
+OptionNumber<std::uint32_t> & ConfigMain::max_downloads_per_mirror() { return pImpl->max_downloads_per_mirror; }
 OptionSeconds & ConfigMain::metadata_expire() { return pImpl->metadata_expire; }
 OptionString & ConfigMain::sslcacert() { return pImpl->sslcacert; }
 OptionBool & ConfigMain::sslverify() { return pImpl->sslverify; }
 OptionString & ConfigMain::sslclientcert() { return pImpl->sslclientcert; }
 OptionString & ConfigMain::sslclientkey() { return pImpl->sslclientkey; }
+OptionString & ConfigMain::proxy_sslcacert() { return pImpl->proxy_sslcacert; }
+OptionBool & ConfigMain::proxy_sslverify() { return pImpl->proxy_sslverify; }
+OptionString & ConfigMain::proxy_sslclientcert() { return pImpl->proxy_sslclientcert; }
+OptionString & ConfigMain::proxy_sslclientkey() { return pImpl->proxy_sslclientkey; }
 OptionBool & ConfigMain::deltarpm() { return pImpl->deltarpm; }
 OptionNumber<std::uint32_t> & ConfigMain::deltarpm_percentage() { return pImpl->deltarpm_percentage; }
 OptionBool & ConfigMain::skip_if_unavailable() { return pImpl->skip_if_unavailable; }
+OptionBool & ConfigMain::sslverifystatus() { return pImpl->sslverifystatus; }
 
 static void DIRClose(DIR *d) { closedir(d); }
 
