@@ -249,6 +249,11 @@ gboolean         dnf_context_update                     (DnfContext     *context
                                                          GError         **error);
 gboolean         dnf_context_update_all                 (DnfContext     *context,
                                                          GError         **error);
+gboolean         dnf_context_distrosync                 (DnfContext     *context,
+                                                         const gchar    *name,
+                                                         GError         **error);
+gboolean         dnf_context_distrosync_all             (DnfContext     *context,
+                                                         GError         **error);
 gboolean         dnf_context_repo_enable                (DnfContext     *context,
                                                          const gchar    *repo_id,
                                                          GError         **error);
@@ -275,21 +280,49 @@ bool             dnf_context_plugin_hook                (DnfContext     *context
                                                          DnfPluginError *error);
 /// String must be dealocated by g_free()
 gchar *          dnf_context_get_module_report          (DnfContext * context);
+
+/**
+ * dnf_context_reset_modules:
+ * @context: DnfContext
+ * @sack: DnfSack
+ * @module_names: Names of modules to reset
+ * @error: Error
+ *
+ * Reset modules, commit modular changes, and recalculate module filtration.
+ * Note you likely want to use dnf_context_module_reset instead which matches
+ * the behaviour of other modular APIs to not commit modular changes to disk
+ * until dnf_context_run(). Returns FALSE when an error is set.
+ *
+ * Since: 0.38.1
+ **/
 gboolean         dnf_context_reset_modules              (DnfContext * context,
                                                          DnfSack * sack,
                                                          const char ** module_names,
                                                          GError ** error);
+
+/**
+ * dnf_context_reset_all_modules:
+ * @context: DnfContext
+ * @sack: DnfSack
+ * @error: Error
+ *
+ * Reset all modules and recalculate module filtration.
+ * Returns FALSE when an error is set.
+ *
+ * Since: 0.46.2
+ **/
 gboolean         dnf_context_reset_all_modules          (DnfContext * context,
                                                          DnfSack * sack,
                                                          GError ** error);
+
 /**
  * dnf_context_module_enable:
  * @context: DnfContext
  * @module_specs: Module specs that should be enabled
  * @error: Error
  *
- * Enable mudules, recalculate module filtration, but do not commit modular changes.
- * To commit modular changes it requires to call dnf_context_run()
+ * Enable modules, recalculate module filtration, but do not commit modular changes.
+ * To commit modular changes, call dnf_context_run().
  * Returns FALSE when an error is set.
  *
  * Since: 0.55.0
@@ -297,13 +330,30 @@ gboolean         dnf_context_reset_all_modules          (DnfContext * context,
 gboolean         dnf_context_module_enable              (DnfContext * context,
                                                          const char ** module_specs,
                                                          GError ** error);
+
+/**
+ * dnf_context_module_install:
+ * @context: DnfContext
+ * @module_specs: Module specs that should be installed
+ * @error: Error
+ *
+ * Enable modules and mark for installation but do not commit modular changes.
+ * To commit modular changes, call dnf_context_run().
+ * Returns FALSE when an error is set.
+ *
+ * Since: 0.63.0
+ **/
+gboolean           dnf_context_module_install            (DnfContext * context,
+                                                          const char ** module_specs,
+                                                          GError ** error);
+
 /**
  * dnf_context_module_disable:
  * @context: DnfContext
  * @module_specs: Module specs that should be enabled
  * @error: Error
  *
- * Disable mudules, recalculate module filtration, but do not commit modular changes.
+ * Disable modules, recalculate module filtration, but do not commit modular changes.
  * To commit modular changes it requires to call dnf_context_run()
  * Returns FALSE when an error is set.
  *
@@ -312,6 +362,22 @@ gboolean         dnf_context_module_enable              (DnfContext * context,
 gboolean         dnf_context_module_disable             (DnfContext * context,
                                                          const char ** module_specs,
                                                          GError ** error);
+
+/**
+ * dnf_context_module_disable_all:
+ * @context: DnfContext
+ * @module_specs: Module specs that should be enabled
+ * @error: Error
+ *
+ * Disable all modules, recalculate module filtration, but do not commit modular
+ * changes.  To commit modular changes it requires to call dnf_context_run()
+ * Returns FALSE when an error is set.
+ *
+ * Since: 0.64.0
+ **/
+gboolean         dnf_context_module_disable_all         (DnfContext * context,
+                                                         GError ** error);
+
 /**
  * dnf_context_module_reset:
  * @context: DnfContext
