@@ -41,6 +41,7 @@
 
 #include <strings.h>
 #include <fcntl.h>
+#include <fnmatch.h>
 #include <glib/gstdio.h>
 #include "hy-util.h"
 #include <librepo/librepo.h>
@@ -55,6 +56,7 @@
 #include "dnf-utils.h"
 #include "utils/File.hpp"
 #include "utils/url-encode.hpp"
+#include "utils/utils.hpp"
 
 #include <set>
 #include <string>
@@ -63,7 +65,6 @@
 typedef struct
 {
     DnfRepoEnabled   enabled;
-    gchar          **gpgkeys;
     gchar          **exclude_packages;
     gchar           *filename;      /* /etc/yum.repos.d/updates.repo */
     gchar           *location;      /* /var/cache/PackageKit/metadata/fedora */
@@ -82,6 +83,7 @@ typedef struct
     LrHandle        *repo_handle;
     LrResult        *repo_result;
     LrUrlVars       *urlvars;
+    bool            unit_test_mode;  /* ugly hack for unit tests */
 } DnfRepoPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE(DnfRepo, dnf_repo, G_TYPE_OBJECT)
@@ -97,7 +99,6 @@ dnf_repo_finalize(GObject *object)
     DnfRepoPrivate *priv = GET_PRIVATE(repo);
 
     g_free(priv->filename);
-    g_strfreev(priv->gpgkeys);
     g_strfreev(priv->exclude_packages);
     g_free(priv->location_tmp);
     g_free(priv->location);
@@ -225,16 +226,13 @@ gchar **
 dnf_repo_get_public_keys(DnfRepo *repo)
 {
     DnfRepoPrivate *priv = GET_PRIVATE(repo);
-    g_autoptr(GPtrArray) ret = g_ptr_array_new();
-    for (char **iter = priv->gpgkeys; iter && *iter; iter++) {
-        const char *key = *iter;
-        g_autofree gchar *key_bn = g_path_get_basename(key);
-        /* transfer ownership to ptrarray */
-        g_ptr_array_add(ret, g_build_filename(priv->location, key_bn, NULL));
+    const auto & keys = priv->repo->getConfig()->gpgkey().getValue();
+    gchar **ret = g_new0(gchar *, keys.size() + 1);
+    for (size_t i = 0; i < keys.size(); ++i) {
+        g_autofree gchar *key_bn = g_path_get_basename(keys[i].c_str());
+        ret[i] = g_build_filename(priv->location, key_bn, NULL);
     }
-    g_ptr_array_add(ret, NULL);
-    /* transfer ownership of container and elements to caller */
-    return (gchar**)g_ptr_array_free(static_cast<GPtrArray *>(g_steal_pointer(&ret)), FALSE);
+    return ret;
 }
 
 /**
@@ -817,211 +815,189 @@ dnf_repo_set_metadata_expire(DnfRepo *repo, guint metadata_expire)
 }
 
 /**
- *  dnf_repo_parse_time_from_str
- *  @expression: a expression to be parsed
- *  @out_parsed_time: (out): return location for parsed time
- *  @error: error item
- *
- *  Parse String into an integer value of seconds, or a human
- *  readable variation specifying days, hours, minutes or seconds
- *  until something happens. Note that due to historical president
- *  -1 means "never", so this accepts that and allows
- *  the word never, too.
- *
- *  Valid inputs: 100, 1.5m, 90s, 1.2d, 1d, 0xF, 0.1, -1, never.
- *  Invalid inputs: -10, -0.1, 45.6Z, 1d6h, 1day, 1y.
-
- *  Returns: integer value in seconds
- **/
-
-static gboolean
-dnf_repo_parse_time_from_str(const gchar *expression, guint *out_parsed_time, GError **error)
+* @brief Format user password string
+*
+* Returns user and password in user:password form. If encode is True,
+* special characters in user and password are URL encoded.
+*
+* @param user Username
+* @param passwd Password
+* @param encode If quote is True, special characters in user and password are URL encoded.
+* @return User and password in user:password form
+*/
+static std::string formatUserPassString(const std::string & user, const std::string & passwd, bool encode)
 {
-    gint multiplier;
-    gdouble parsed_time;
-    gchar *endptr = NULL;
-
-    if (!g_strcmp0(expression, "")) {
-        g_set_error_literal(error,
-                            DNF_ERROR,
-                            DNF_ERROR_FILE_INVALID,
-                            "no metadata value specified");
-        return FALSE;
-    }
-
-    if (g_strcmp0(expression, "-1") == 0 ||
-        g_strcmp0(expression,"never") == 0) {
-        *out_parsed_time = G_MAXUINT;
-        return TRUE; /* Note early return */
-    }
-
-    gchar last_char = expression[ strlen(expression) - 1 ];
-
-    /* check if the input ends with h, m ,d ,s as units */
-    if (g_ascii_isalpha(last_char)) {
-        if (last_char == 'h')
-            multiplier = 60 * 60;
-        else if (last_char == 's')
-            multiplier = 1;
-        else if (last_char == 'm')
-            multiplier = 60;
-        else if (last_char == 'd')
-            multiplier = 60 * 60 * 24;
-        else {
-            g_set_error(error, DNF_ERROR, DNF_ERROR_FILE_INVALID,
-                        "unknown unit %c", last_char);
-            return FALSE;
-        }
-    }
+    if (encode)
+        return libdnf::urlEncode(user) + ":" + libdnf::urlEncode(passwd);
     else
-        multiplier = 1;
-
-    /* convert expression into a double*/
-    parsed_time = g_ascii_strtod(expression, &endptr);
-
-    /* failed to parse */
-    if (expression == endptr) {
-        g_set_error(error, DNF_ERROR, DNF_ERROR_INTERNAL_ERROR,
-                    "failed to parse time: %s", expression);
-        return FALSE;
-    }
-
-    /* time can not be below zero */
-    if (parsed_time < 0) {
-        g_set_error(error, DNF_ERROR, DNF_ERROR_INTERNAL_ERROR,
-                    "seconds value must not be negative %s",expression );
-        return FALSE;
-    }
-
-    /* time too large */
-    if (parsed_time > G_MAXDOUBLE || (parsed_time * multiplier) > G_MAXUINT){
-        g_set_error(error, DNF_ERROR, DNF_ERROR_INTERNAL_ERROR,
-                    "time too large");
-        return FALSE;
-    }
-
-    *out_parsed_time = (guint) (parsed_time * multiplier);
-    return TRUE;
-}
-/**
- * dnf_repo_get_username_password_string:
- */
-static gchar *
-dnf_repo_get_username_password_string(const gchar *user, const gchar *pass)
-{
-    if (user == NULL && pass == NULL)
-        return NULL;
-    if (user != NULL && pass == NULL)
-        return g_strdup(user);
-    if (user == NULL && pass != NULL)
-        return g_strdup_printf(":%s", pass);
-    return g_strdup_printf("%s:%s", user, pass);
+        return user + ":" + passwd;
 }
 
-static gboolean
-dnf_repo_get_boolean(GKeyFile *keyfile,
-                     const gchar *group_name,
-                     const gchar *key)
+/* Resets repository configuration options previously readed from repository
+ * configuration file to initial state. */
+static void
+dnf_repo_conf_reset(libdnf::ConfigRepo &config)
 {
-    g_autofree gchar *str = NULL;
-
-    str = g_key_file_get_value(keyfile, group_name, key, NULL);
-    if (str) {
-        try {
-            return libdnf::OptionBool(false).fromString(str);
-        } catch (const std::exception & ex) {
-            g_warning("Config error in section \"%s\" key \"%s\": %s", group_name, key, ex.what());
+    for (auto & item : config.optBinds()) {
+        auto & itemOption = item.second;
+        if (itemOption.getPriority() == libdnf::Option::Priority::REPOCONFIG) {
+            itemOption.getOption().reset();
+        }
+    }
+}
+
+/* Loads repository configuration from GKeyFile */
+static void
+dnf_repo_conf_from_gkeyfile(DnfRepo *repo, const char *repoId, GKeyFile *gkeyFile)
+{
+    DnfRepoPrivate *priv = GET_PRIVATE(repo);
+    auto & config = *priv->repo->getConfig();
+
+    // Reset to the initial state before reloading the configuration.
+    dnf_repo_conf_reset(config);
+
+    g_autoptr(GError) error_local = NULL;
+    g_auto(GStrv) keys = g_key_file_get_keys(gkeyFile, repoId, NULL, &error_local);
+    if (keys == NULL) {
+        g_debug("Failed to load configuration for repo id \"%s\": %s", repoId, error_local->message);
+        return;
+    }
+
+    for (auto it = keys; *it != NULL; ++it) {
+        auto key = *it;
+        g_autofree gchar *str = g_key_file_get_value(gkeyFile, repoId, key, NULL);
+        if (str) {
+            auto value = libdnf::string::trim(str);
+            if (value.length() > 1 && value.front() == value.back() &&
+                (value.front() == '"' || value.front() == '\'')) {
+                value.erase(--value.end());
+                value.erase(value.begin());
+            }
+            try {
+                auto & optionItem = config.optBinds().at(key);
+
+                if (dynamic_cast<libdnf::OptionStringList*>(&optionItem.getOption()) ||
+                    dynamic_cast<libdnf::OptionChild<libdnf::OptionStringList>*>(&optionItem.getOption())
+                ) {
+
+                    // reload list option from gKeyFile using g_key_file_get_string_list()
+                    // g_key_file_get_value () is problematic for multiline lists
+                    g_auto(GStrv) list = g_key_file_get_string_list(gkeyFile, repoId, key, NULL, NULL);
+                    if (list) {
+                        // list can be ['value1', 'value2, value3'] therefore we first join
+                        // to have 'value1, value2, value3'
+                        g_autofree gchar * tmp_strval = g_strjoinv(",", list);
+
+                        // Substitute vars.
+                        g_autofree gchar *subst_value = dnf_repo_substitute(repo, tmp_strval);
+
+                        if (strcmp(key, "baseurl") == 0 && strstr(tmp_strval, "file://$testdatadir") != NULL) {
+                            priv->unit_test_mode = true;
+                        }
+
+                        try {
+                            optionItem.newString(libdnf::Option::Priority::REPOCONFIG, subst_value);
+                        } catch (const std::exception & ex) {
+                            g_debug("Invalid configuration value: %s = %s in %s; %s", key, subst_value, repoId, ex.what());
+                        }
+                    }
+
+                } else {
+                    // process other (non list) options
+
+                    // Substitute vars.
+                    g_autofree gchar *subst_value = dnf_repo_substitute(repo, value.c_str());
+
+                    try {
+                        optionItem.newString(libdnf::Option::Priority::REPOCONFIG, subst_value);
+                    } catch (const std::exception & ex) {
+                        g_debug("Invalid configuration value: %s = %s in %s; %s", key, subst_value, repoId, ex.what());
+                    }
+
+                }
+
+            } catch (const std::exception &) {
+                g_debug("Unknown configuration option: %s = %s in %s", key, value.c_str(), repoId);
+            }
+        }
+    }
+}
+
+static void
+dnf_repo_apply_setopts(libdnf::ConfigRepo &config, const char *repoId)
+{
+    // apply repository setopts
+    auto & optBinds = config.optBinds();
+    for (const auto & setopt : libdnf::getGlobalSetopts()) {
+        auto last_dot_pos = setopt.key.rfind('.');
+        if (last_dot_pos == std::string::npos) {
+            continue;
+        }
+        auto repo_pattern = setopt.key.substr(0, last_dot_pos);
+        if (fnmatch(repo_pattern.c_str(), repoId, FNM_CASEFOLD) == 0) {
+            auto key = setopt.key.substr(last_dot_pos+1);
+            try {
+                auto & optionItem = optBinds.at(key);
+                try {
+                    optionItem.newString(setopt.priority, setopt.value);
+                } catch (const std::exception & ex) {
+                    g_warning("dnf_repo_apply_setopt: Invalid configuration value: %s = %s; %s", setopt.key.c_str(), setopt.value.c_str(), ex.what());
+                }
+            } catch (const std::exception &) {
+                g_warning("dnf_repo_apply_setopt: Unknown configuration option: %s in %s = %s", key.c_str(), setopt.key.c_str(), setopt.value.c_str());
+            }
         }
     }
-    return false;
 }
 
 /* Initialize (or potentially reset) repo & LrHandle from keyfile values. */
 static gboolean
-dnf_repo_set_keyfile_data(DnfRepo *repo, GError **error)
+dnf_repo_set_keyfile_data(DnfRepo *repo, gboolean reloadFromGKeyFile, GError **error)
 {
     DnfRepoPrivate *priv = GET_PRIVATE(repo);
-    guint cost;
-    gboolean module_hotfixes = false;
-    g_autofree gchar *metadata_expire_str = NULL;
-    g_autofree gchar *mirrorlist = NULL;
-    g_autofree gchar *mirrorlisturl = NULL;
-    g_autofree gchar *metalinkurl = NULL;
-    g_autofree gchar *proxy = NULL;
-    g_autofree gchar *pwd = NULL;
-    g_autofree gchar *usr = NULL;
-    g_autofree gchar *usr_pwd = NULL;
-    g_autofree gchar *usr_pwd_proxy = NULL;
-    g_auto(GStrv) baseurls;
+    std::string tmp_str;
+    const char *tmp_cstr;
 
     auto repoId = priv->repo->getId().c_str();
     g_debug("setting keyfile data for %s", repoId);
 
-    /* skip_if_unavailable is optional */
-    if (g_key_file_has_key(priv->keyfile, repoId, "skip_if_unavailable", NULL)) {
-        bool skip = dnf_repo_get_boolean(priv->keyfile, repoId, "skip_if_unavailable");
-        priv->repo->getConfig()->skip_if_unavailable().set(libdnf::Option::Priority::REPOCONFIG, skip);
+    auto conf = priv->repo->getConfig();
+
+    // Reload repository configuration from keyfile.
+    if (reloadFromGKeyFile) {
+        dnf_repo_conf_from_gkeyfile(repo, repoId, priv->keyfile);
+        dnf_repo_apply_setopts(*conf, repoId);
     }
 
-    /* priority is optional */
-    g_autofree gchar * priority_str = g_key_file_get_string(priv->keyfile, repoId, "priority", NULL);
-    if (priority_str) {
-        priv->repo->getConfig()->priority().set(libdnf::Option::Priority::REPOCONFIG, priority_str);
-    }
-
-    /* cost is optional */
-    cost = g_key_file_get_integer(priv->keyfile, repoId, "cost", NULL);
-    if (cost != 0)
-        dnf_repo_set_cost(repo, cost);
-
-    module_hotfixes = g_key_file_get_boolean(priv->keyfile, repoId, "module_hotfixes", NULL);
-    priv->repo->getConfig()->module_hotfixes().set(libdnf::Option::Priority::REPOCONFIG, module_hotfixes);
-
     /* baseurl is optional; if missing, unset it */
-    baseurls = g_key_file_get_string_list(priv->keyfile, repoId, "baseurl", NULL, NULL);
-    if (baseurls) {
-        // baseruls can be ['value1', 'value2, value3'] therefore we first join to have 'value1, value2, value3'
-        g_autofree gchar * tmp_strval = g_strjoinv(",", baseurls);
-
-        auto & bindBaseurls = priv->repo->getConfig()->optBinds().at("baseurl");
-        bindBaseurls.newString(libdnf::Option::Priority::REPOCONFIG, tmp_strval);
-
-        auto & repoBaseurls = priv->repo->getConfig()->baseurl();
-        if (!repoBaseurls.getValue().empty()){
-            auto len = repoBaseurls.getValue().size();
-            g_strfreev(baseurls);
-            baseurls = g_new0(char *, len + 1);
-            for (size_t i = 0; i < len; ++i) {
-                baseurls[i] = g_strdup(repoBaseurls.getValue()[i].c_str());
-            }
+    g_auto(GStrv) baseurls = NULL;
+    auto & repoBaseurls = conf->baseurl().getValue();
+    if (!repoBaseurls.empty()){
+        auto len = repoBaseurls.size();
+        baseurls = g_new0(char *, len + 1);
+        for (size_t i = 0; i < len; ++i) {
+            baseurls[i] = g_strdup(repoBaseurls[i].c_str());
         }
     }
     if (!lr_handle_setopt(priv->repo_handle, error, LRO_URLS, baseurls))
         return FALSE;
 
-    /* metadata_expire is optional, if shown, we parse the string to add the time */
-    metadata_expire_str = g_key_file_get_string(priv->keyfile, repoId, "metadata_expire", NULL);
-    if (metadata_expire_str) {
-        guint metadata_expire;
-        if (!dnf_repo_parse_time_from_str(metadata_expire_str, &metadata_expire, error))
-            return FALSE;
-        dnf_repo_set_metadata_expire(repo, metadata_expire);
-    }
+    const char *mirrorlisturl = NULL;
+    const char *metalinkurl = NULL;
 
     /* the "mirrorlist" entry could be either a real mirrorlist, or a metalink entry */
-    mirrorlist = g_key_file_get_string(priv->keyfile, repoId, "mirrorlist", NULL);
-    if (mirrorlist) {
-        if (strstr(mirrorlist, "metalink"))
-            metalinkurl = static_cast<gchar *>(g_steal_pointer(&mirrorlist));
+    tmp_cstr = conf->mirrorlist().empty() ? NULL : conf->mirrorlist().getValue().c_str();
+    if (tmp_cstr) {
+        if (strstr(tmp_cstr, "metalink"))
+            metalinkurl = tmp_cstr;
         else /* it really is a mirrorlist */
-            mirrorlisturl = static_cast<gchar *>(g_steal_pointer(&mirrorlist));
+            mirrorlisturl = tmp_cstr;
     }
 
     /* let "metalink" entry override metalink-as-mirrorlist entry */
-    if (g_key_file_has_key(priv->keyfile, repoId, "metalink", NULL)) {
-        g_free(metalinkurl);
-        metalinkurl = g_key_file_get_string(priv->keyfile, repoId, "metalink", NULL);
-    }
+    tmp_cstr = conf->metalink().empty() ? NULL : conf->metalink().getValue().c_str();
+    if (tmp_cstr)
+        metalinkurl = tmp_cstr;
 
     /* now set the final values (or unset them) */
     if (!lr_handle_setopt(priv->repo_handle, error, LRO_MIRRORLISTURL, mirrorlisturl))
@@ -1029,26 +1005,15 @@ dnf_repo_set_keyfile_data(DnfRepo *repo, GError **error)
     if (!lr_handle_setopt(priv->repo_handle, error, LRO_METALINKURL, metalinkurl))
         return FALSE;
 
-    /* needed in order for addCountmeFlag() to use the same persistdir as DNF
-     * would */
-    if (metalinkurl)
-        priv->repo->getConfig()->metalink().set(libdnf::Option::Priority::REPOCONFIG, metalinkurl);
-    if (mirrorlisturl)
-        priv->repo->getConfig()->mirrorlist().set(libdnf::Option::Priority::REPOCONFIG, mirrorlisturl);
-
-    if (g_key_file_has_key(priv->keyfile, repoId, "countme", NULL)) {
-        bool countme = dnf_repo_get_boolean(priv->keyfile, repoId, "countme");
-        priv->repo->getConfig()->countme().set(libdnf::Option::Priority::REPOCONFIG, countme);
-    }
-
     /* file:// */
     if (baseurls != NULL && baseurls[0] != NULL &&
         mirrorlisturl == NULL && metalinkurl == NULL) {
         g_autofree gchar *url = NULL;
         url = lr_prepend_url_protocol(baseurls[0]);
         if (url != NULL && strncasecmp(url, "file://", 7) == 0) {
-            if (g_strstr_len(url, -1, "$testdatadir") == NULL)
+            if (!priv->unit_test_mode) {
                 priv->kind = DNF_REPO_KIND_LOCAL;
+            }
             g_free(priv->location);
             g_free(priv->keyring);
             priv->location = dnf_repo_substitute(repo, url + 7);
@@ -1087,43 +1052,8 @@ dnf_repo_set_keyfile_data(DnfRepo *repo, GError **error)
     }
 
     /* gpgkey is optional for gpgcheck=1, but required for repo_gpgcheck=1 */
-    g_strfreev(priv->gpgkeys);
-    priv->gpgkeys = NULL;
-
-    g_auto(GStrv) gpgkeys;
-    gpgkeys = g_key_file_get_string_list(priv->keyfile, repoId, "gpgkey", NULL, NULL);
-
-    if (gpgkeys) {
-        // gpgkeys can be ['value1', 'value2, value3'] therefore we first join to have 'value1, value2, value3'
-        g_autofree gchar * tmp_strval = g_strjoinv(",", gpgkeys);
-
-        auto & bindGpgkeys = priv->repo->getConfig()->optBinds().at("gpgkey");
-        bindGpgkeys.newString(libdnf::Option::Priority::REPOCONFIG, tmp_strval);
-
-        auto & repoGpgkeys = priv->repo->getConfig()->gpgkey();
-        if (!repoGpgkeys.getValue().empty()){
-            auto len = repoGpgkeys.getValue().size();
-            priv->gpgkeys = g_new0(char *, len + 1);
-            for (size_t i = 0; i < len; ++i) {
-                priv->gpgkeys[i] = g_strdup(repoGpgkeys.getValue()[i].c_str());
-            }
-        } else {
-            /* Canonicalize the empty list to NULL for ease of checking elsewhere */
-            g_strfreev(static_cast<gchar **>(g_steal_pointer(&priv->gpgkeys)));
-        }
-    }
-
-    if (g_key_file_has_key(priv->keyfile, repoId, "gpgcheck", NULL)) {
-        auto gpgcheck_pkgs = dnf_repo_get_boolean(priv->keyfile, repoId, "gpgcheck");
-        priv->repo->getConfig()->gpgcheck().set(libdnf::Option::Priority::REPOCONFIG, gpgcheck_pkgs);
-    }
-
-    if (g_key_file_has_key(priv->keyfile, repoId, "repo_gpgcheck", NULL)) {
-        auto gpgcheck_md = dnf_repo_get_boolean(priv->keyfile, repoId, "repo_gpgcheck");
-        priv->repo->getConfig()->repo_gpgcheck().set(libdnf::Option::Priority::REPOCONFIG, gpgcheck_md);
-    }
-    auto gpgcheck_md = priv->repo->getConfig()->repo_gpgcheck().getValue();
-    if (gpgcheck_md && priv->gpgkeys == NULL) {
+    auto repo_gpgcheck = conf->repo_gpgcheck().getValue();
+    if (repo_gpgcheck && conf->gpgkey().getValue().empty()) {
         g_set_error_literal(error,
                             DNF_ERROR,
                             DNF_ERROR_FILE_INVALID,
@@ -1132,58 +1062,114 @@ dnf_repo_set_keyfile_data(DnfRepo *repo, GError **error)
     }
 
     /* XXX: setopt() expects a long, so we need a long on the stack */
-    if (!lr_handle_setopt(priv->repo_handle, error, LRO_GPGCHECK, (long)gpgcheck_md))
+    if (!lr_handle_setopt(priv->repo_handle, error, LRO_GPGCHECK, (long)repo_gpgcheck))
         return FALSE;
 
-    auto & repoExcludepkgs = priv->repo->getConfig()->excludepkgs();
-    repoExcludepkgs.set(libdnf::Option::Priority::REPOCONFIG, "");
-
-    auto & bindExcludepkgs = priv->repo->getConfig()->optBinds().at("excludepkgs");
-    if (auto excludepkgs = g_key_file_get_string(priv->keyfile, repoId, "exclude", NULL)) {
-        bindExcludepkgs.newString(libdnf::Option::Priority::REPOCONFIG, excludepkgs);
-        g_free(excludepkgs);
-    }
-    if (auto excludepkgs = g_key_file_get_string(priv->keyfile, repoId, "excludepkgs", NULL)) {
-        bindExcludepkgs.newString(libdnf::Option::Priority::REPOCONFIG, excludepkgs);
-        g_free(excludepkgs);
-    }
-
+    // Sync priv->exclude_packages
     g_strfreev(priv->exclude_packages);
-    if (!repoExcludepkgs.getValue().empty()) {
-        auto len = repoExcludepkgs.getValue().size();
+    auto & repoExcludepkgs = conf->excludepkgs().getValue();
+    if (!repoExcludepkgs.empty()) {
+        auto len = repoExcludepkgs.size();
         priv->exclude_packages = g_new0(char *, len + 1);
         for (size_t i = 0; i < len; ++i) {
-            priv->exclude_packages[i] = g_strdup(repoExcludepkgs.getValue()[i].c_str());
+            priv->exclude_packages[i] = g_strdup(repoExcludepkgs[i].c_str());
         }
     } else {
         priv->exclude_packages = NULL;
     }
 
-    if (auto includepkgs = g_key_file_get_string(priv->keyfile, repoId, "includepkgs", NULL)) {
-        priv->repo->getConfig()->includepkgs().set(libdnf::Option::Priority::REPOCONFIG, includepkgs);
-        g_free(includepkgs);
+    auto minrate = conf->minrate().getValue();
+    if (!lr_handle_setopt(priv->repo_handle, error, LRO_LOWSPEEDLIMIT, static_cast<long>(minrate)))
+        return FALSE;
+
+    auto maxspeed = conf->throttle().getValue();
+    if (maxspeed > 0 && maxspeed <= 1)
+        maxspeed *= conf->bandwidth().getValue();
+    if (maxspeed != 0 && maxspeed < minrate) {
+        g_set_error_literal(error, DNF_ERROR, DNF_ERROR_FILE_INVALID,
+                            "Maximum download speed is lower than minimum. "
+                            "Please change configuration of minrate or throttle");
+        return FALSE;
+    }
+    if (!lr_handle_setopt(priv->repo_handle, error, LRO_MAXSPEED, static_cast<int64_t>(maxspeed)))
+        return FALSE;
+
+    long timeout = conf->timeout().getValue();
+    if (timeout > 0) {
+        if (!lr_handle_setopt(priv->repo_handle, error, LRO_CONNECTTIMEOUT, timeout))
+            return FALSE;
+        if (!lr_handle_setopt(priv->repo_handle, error, LRO_LOWSPEEDTIME, timeout))
+            return FALSE;
+    } else {
+        if (!lr_handle_setopt(priv->repo_handle, error, LRO_CONNECTTIMEOUT, LRO_CONNECTTIMEOUT_DEFAULT))
+            return FALSE;
+        if (!lr_handle_setopt(priv->repo_handle, error, LRO_LOWSPEEDTIME, LRO_LOWSPEEDTIME_DEFAULT))
+            return FALSE;
     }
 
-    /* proxy is optional */
-    proxy = g_key_file_get_string(priv->keyfile, repoId, "proxy", NULL);
-    auto repoProxy = proxy ? (strcasecmp(proxy, "_none_") == 0 ? NULL : proxy)
-        : dnf_context_get_http_proxy(priv->context);
-    if (!lr_handle_setopt(priv->repo_handle, error, LRO_PROXY, repoProxy))
+    tmp_str = conf->proxy().getValue();
+    tmp_cstr = tmp_str.empty() ? dnf_context_get_http_proxy(priv->context) : tmp_str.c_str();
+    if (!lr_handle_setopt(priv->repo_handle, error, LRO_PROXY, tmp_cstr))
         return FALSE;
 
-    /* both parts of the proxy auth are optional */
-    usr = g_key_file_get_string(priv->keyfile, repoId, "proxy_username", NULL);
-    pwd = g_key_file_get_string(priv->keyfile, repoId, "proxy_password", NULL);
-    usr_pwd_proxy = dnf_repo_get_username_password_string(usr, pwd);
-    if (!lr_handle_setopt(priv->repo_handle, error, LRO_PROXYUSERPWD, usr_pwd_proxy))
+    // setup proxy authorization method
+    auto proxyAuthMethods = libdnf::Repo::Impl::stringToProxyAuthMethods(conf->proxy_auth_method().getValue());
+    if (!lr_handle_setopt(priv->repo_handle, error, LRO_PROXYAUTHMETHODS, static_cast<long>(proxyAuthMethods)))
         return FALSE;
 
-    /* both parts of the HTTP auth are optional */
-    usr = g_key_file_get_string(priv->keyfile, repoId, "username", NULL);
-    pwd = g_key_file_get_string(priv->keyfile, repoId, "password", NULL);
-    usr_pwd = dnf_repo_get_username_password_string(usr, pwd);
-    if (!lr_handle_setopt(priv->repo_handle, error, LRO_USERPWD, usr_pwd))
+    // setup proxy username and password
+    tmp_cstr = NULL;
+    if (!conf->proxy_username().empty()) {
+        tmp_str = conf->proxy_username().getValue();
+        if (!tmp_str.empty()) {
+            if (conf->proxy_password().empty()) {
+                g_set_error(error, DNF_ERROR, DNF_ERROR_FILE_INVALID,
+                            "repo '%s': 'proxy_username' is set but not 'proxy_password'", repoId);
+                return FALSE;
+            }
+            tmp_str = formatUserPassString(tmp_str, conf->proxy_password().getValue(), true);
+            tmp_cstr = tmp_str.c_str();
+        }
+    }
+    if (!lr_handle_setopt(priv->repo_handle, error, LRO_PROXYUSERPWD, tmp_cstr))
         return FALSE;
+
+    // setup username and password
+    tmp_cstr = NULL;
+    tmp_str = conf->username().getValue();
+    if (!tmp_str.empty()) {
+        // TODO Use URL encoded form, needs support in librepo
+        tmp_str = formatUserPassString(tmp_str, conf->password().getValue(), false);
+        tmp_cstr = tmp_str.c_str();
+    }
+    if (!lr_handle_setopt(priv->repo_handle, error, LRO_USERPWD, tmp_cstr))
+        return FALSE;
+
+    auto proxy_sslverify = conf->proxy_sslverify().getValue();
+    /* XXX: setopt() expects a long, so we need a long on the stack */
+    if (!lr_handle_setopt(priv->repo_handle, error, LRO_PROXY_SSLVERIFYPEER, (long)proxy_sslverify))
+        return FALSE;
+    if (!lr_handle_setopt(priv->repo_handle, error, LRO_PROXY_SSLVERIFYHOST, (long)proxy_sslverify))
+        return FALSE;
+
+    auto & proxy_sslcacert = conf->proxy_sslcacert().getValue();
+    if (!proxy_sslcacert.empty()) {
+        if (!lr_handle_setopt(priv->repo_handle, error, LRO_PROXY_SSLCACERT, proxy_sslcacert.c_str()))
+            return FALSE;
+    }
+
+    auto & proxy_sslclientcert = conf->proxy_sslclientcert().getValue();
+    if (!proxy_sslclientcert.empty()) {
+        if (!lr_handle_setopt(priv->repo_handle, error, LRO_PROXY_SSLCLIENTCERT, proxy_sslclientcert.c_str()))
+            return FALSE;
+    }
+
+    auto & proxy_sslclientkey = conf->proxy_sslclientkey().getValue();
+    if (!proxy_sslclientkey.empty()) {
+        if (!lr_handle_setopt(priv->repo_handle, error, LRO_PROXY_SSLCLIENTKEY, proxy_sslclientkey.c_str()))
+            return FALSE;
+    }
+
     return TRUE;
 }
 
@@ -1205,11 +1191,6 @@ dnf_repo_setup(DnfRepo *repo, GError **error) try
     DnfRepoEnabled enabled = DNF_REPO_ENABLED_NONE;
     g_autofree gchar *basearch = NULL;
     g_autofree gchar *release = NULL;
-    g_autofree gchar *testdatadir = NULL;
-    g_autofree gchar *sslcacert = NULL;
-    g_autofree gchar *sslclientcert = NULL;
-    g_autofree gchar *sslclientkey = NULL;
-    gboolean sslverify = TRUE;
 
     basearch = g_key_file_get_string(priv->keyfile, "general", "arch", NULL);
     if (basearch == NULL)
@@ -1248,41 +1229,45 @@ dnf_repo_setup(DnfRepo *repo, GError **error) try
     for (const auto & item : libdnf::dnf_context_get_vars(priv->context))
         priv->urlvars = lr_urlvars_set(priv->urlvars, item.first.c_str(), item.second.c_str());
 
-    testdatadir = dnf_realpath(TESTDATADIR);
-    priv->urlvars = lr_urlvars_set(priv->urlvars, "testdatadir", testdatadir);
     if (!lr_handle_setopt(priv->repo_handle, error, LRO_VARSUB, priv->urlvars))
         return FALSE;
     if (!lr_handle_setopt(priv->repo_handle, error, LRO_GNUPGHOMEDIR, priv->keyring))
         return FALSE;
 
     auto repoId = priv->repo->getId().c_str();
-    if (g_key_file_has_key(priv->keyfile, repoId, "sslverify", NULL))
-        sslverify = dnf_repo_get_boolean(priv->keyfile, repoId, "sslverify");
 
+    auto conf = priv->repo->getConfig();
+    dnf_repo_conf_from_gkeyfile(repo, repoId, priv->keyfile);
+    dnf_repo_apply_setopts(*conf, repoId);
+
+    auto sslverify = conf->sslverify().getValue();
     /* XXX: setopt() expects a long, so we need a long on the stack */
     if (!lr_handle_setopt(priv->repo_handle, error, LRO_SSLVERIFYPEER, (long)sslverify))
         return FALSE;
     if (!lr_handle_setopt(priv->repo_handle, error, LRO_SSLVERIFYHOST, (long)sslverify))
         return FALSE;
 
-    sslcacert = g_key_file_get_string(priv->keyfile, repoId, "sslcacert", NULL);
-    if (sslcacert != NULL) {
-        if (!lr_handle_setopt(priv->repo_handle, error, LRO_SSLCACERT, sslcacert))
+    auto & sslcacert = conf->sslcacert().getValue();
+    if (!sslcacert.empty()) {
+        if (!lr_handle_setopt(priv->repo_handle, error, LRO_SSLCACERT, sslcacert.c_str()))
             return FALSE;
     }
 
-    sslclientcert = g_key_file_get_string(priv->keyfile, repoId, "sslclientcert", NULL);
-    if (sslclientcert != NULL) {
-        if (!lr_handle_setopt(priv->repo_handle, error, LRO_SSLCLIENTCERT, sslclientcert))
+    auto & sslclientcert = conf->sslclientcert().getValue();
+    if (!sslclientcert.empty()) {
+        if (!lr_handle_setopt(priv->repo_handle, error, LRO_SSLCLIENTCERT, sslclientcert.c_str()))
             return FALSE;
     }
 
-    sslclientkey = g_key_file_get_string(priv->keyfile, repoId, "sslclientkey", NULL);
-    if (sslclientkey != NULL) {
-        if (!lr_handle_setopt(priv->repo_handle, error, LRO_SSLCLIENTKEY, sslclientkey))
+    auto & sslclientkey = conf->sslclientkey().getValue();
+    if (!sslclientkey.empty()) {
+        if (!lr_handle_setopt(priv->repo_handle, error, LRO_SSLCLIENTKEY, sslclientkey.c_str()))
             return FALSE;
     }
 
+    if (!lr_handle_setopt(priv->repo_handle, error, LRO_SSLVERIFYSTATUS, conf->sslverifystatus().getValue() ? 1L : 0L))
+        return FALSE;
+
 #ifdef LRO_SUPPORTS_CACHEDIR
     /* Set cache dir */
     if (dnf_context_get_zchunk(priv->context)) {
@@ -1291,18 +1276,17 @@ dnf_repo_setup(DnfRepo *repo, GError **error) try
     }
 #endif
 
-    /* enabled is optional */
-    if (g_key_file_has_key(priv->keyfile, repoId, "enabled", NULL)) {
-        if (dnf_repo_get_boolean(priv->keyfile, repoId, "enabled"))
-            enabled |= DNF_REPO_ENABLED_PACKAGES;
-    } else {
+    if (conf->enabled().getValue())
         enabled |= DNF_REPO_ENABLED_PACKAGES;
-    }
 
     /* enabled_metadata is optional */
-    if (g_key_file_has_key(priv->keyfile, repoId, "enabled_metadata", NULL)) {
-        if (dnf_repo_get_boolean(priv->keyfile, repoId, "enabled_metadata"))
-            enabled |= DNF_REPO_ENABLED_METADATA;
+    if (conf->enabled_metadata().getPriority() != libdnf::Option::Priority::DEFAULT) {
+        try {
+            if (libdnf::OptionBool(false).fromString(conf->enabled_metadata().getValue()))
+                enabled |= DNF_REPO_ENABLED_METADATA;
+        } catch (const std::exception & ex) {
+            g_warning("Config error in section \"%s\" key \"%s\": %s", repoId, "enabled_metadata", ex.what());
+        }
     } else {
         g_autofree gchar *basename = g_path_get_basename(priv->filename);
         /* special case the satellite and subscription manager repo */
@@ -1312,7 +1296,7 @@ dnf_repo_setup(DnfRepo *repo, GError **error) try
 
     dnf_repo_set_enabled(repo, enabled);
 
-    return dnf_repo_set_keyfile_data(repo, error);
+    return dnf_repo_set_keyfile_data(repo, FALSE, error);
 } CATCH_TO_GERROR(FALSE)
 
 typedef struct
@@ -1530,7 +1514,7 @@ dnf_repo_check_internal(DnfRepo *repo,
         }
     }
     /* ensure we reset the values from the keyfile */
-    if (!dnf_repo_set_keyfile_data(repo, error))
+    if (!dnf_repo_set_keyfile_data(repo, TRUE, error))
         return FALSE;
 
     return TRUE;
@@ -1778,7 +1762,7 @@ dnf_repo_update(DnfRepo *repo,
     }
 
     /* ensure we set the values from the keyfile */
-    if (!dnf_repo_set_keyfile_data(repo, error))
+    if (!dnf_repo_set_keyfile_data(repo, TRUE, error))
         return FALSE;
 
     /* countme support */
@@ -1818,28 +1802,31 @@ dnf_repo_update(DnfRepo *repo,
         goto out;
     }
 
-    if (priv->gpgkeys &&
-        (priv->repo->getConfig()->repo_gpgcheck().getValue() || priv->repo->getConfig()->gpgcheck().getValue())) {
-        for (char **iter = priv->gpgkeys; iter && *iter; iter++) {
-            const char *gpgkey = *iter;
-            g_autofree char *gpgkey_name = g_path_get_basename(gpgkey);
-            g_autofree char *key_tmp = g_build_filename(priv->location_tmp, gpgkey_name, NULL);
+    {
+        const auto & gpgkeys = priv->repo->getConfig()->gpgkey().getValue();
+        if (!gpgkeys.empty() &&
+            (priv->repo->getConfig()->repo_gpgcheck().getValue() || priv->repo->getConfig()->gpgcheck().getValue())) {
+            for (const auto & key : gpgkeys) {
+                const char *gpgkey = key.c_str();
+                g_autofree char *gpgkey_name = g_path_get_basename(gpgkey);
+                g_autofree char *key_tmp = g_build_filename(priv->location_tmp, gpgkey_name, NULL);
 
-            /* download and import public key */
-            if ((g_str_has_prefix(gpgkey, "https://") ||
-                  g_str_has_prefix(gpgkey, "file://"))) {
-                g_debug("importing public key %s", gpgkey);
+                /* download and import public key */
+                if ((g_str_has_prefix(gpgkey, "https://") ||
+                    g_str_has_prefix(gpgkey, "file://"))) {
+                    g_debug("importing public key %s", gpgkey);
 
-                ret = dnf_repo_download_import_public_key(repo, gpgkey, key_tmp, error);
-                if (!ret)
-                    goto out;
-            }
+                    ret = dnf_repo_download_import_public_key(repo, gpgkey, key_tmp, error);
+                    if (!ret)
+                        goto out;
+                }
 
-            /* do we autoimport this into librpm? */
-            if ((flags & DNF_REPO_UPDATE_FLAG_IMPORT_PUBKEY) > 0) {
-                ret = dnf_repo_add_public_key(repo, key_tmp, error);
-                if (!ret)
-                    goto out;
+                /* do we autoimport this into librpm? */
+                if ((flags & DNF_REPO_UPDATE_FLAG_IMPORT_PUBKEY) > 0) {
+                    ret = dnf_repo_add_public_key(repo, key_tmp, error);
+                    if (!ret)
+                        goto out;
+                }
             }
         }
     }
@@ -1872,9 +1859,9 @@ dnf_repo_update(DnfRepo *repo,
 
     /* see dnf_repo_check_internal */
     if (!dnf_context_get_enable_filelists(priv->context)) {
-        const gchar *blacklist[] = { "filelists", NULL };
+        const gchar *excluded_metadata_types[] = { "filelists", NULL };
         ret = lr_handle_setopt(priv->repo_handle, error,
-                               LRO_YUMBLIST, blacklist);
+                               LRO_YUMBLIST, excluded_metadata_types);
         if (!ret)
             goto out;
     }
@@ -1992,9 +1979,12 @@ out:
     g_free(updatedata.last_mirror_failure_message);
     g_free(updatedata.last_mirror_url);
     dnf_state_release_locks(state);
-    lr_handle_setopt(priv->repo_handle, NULL, LRO_PROGRESSCB, NULL);
-    lr_handle_setopt(priv->repo_handle, NULL, LRO_HMFCB, NULL);
-    lr_handle_setopt(priv->repo_handle, NULL, LRO_PROGRESSDATA, 0xdeadbeef);
+    if (!lr_handle_setopt(priv->repo_handle, NULL, LRO_PROGRESSCB, NULL))
+            g_debug("Failed to reset LRO_PROGRESSCB to NULL");
+    if (!lr_handle_setopt(priv->repo_handle, NULL, LRO_HMFCB, NULL))
+            g_debug("Failed to reset LRO_HMFCB to NULL");
+    if (!lr_handle_setopt(priv->repo_handle, NULL, LRO_PROGRESSDATA, 0xdeadbeef))
+            g_debug("Failed to set LRO_PROGRESSDATA to 0xdeadbeef");
     return ret;
 } CATCH_TO_GERROR(FALSE)
 
@@ -2234,7 +2224,7 @@ dnf_repo_download_packages(DnfRepo *repo,
     g_autofree gchar *directory_slash = NULL;
 
     /* ensure we reset the values from the keyfile */
-    if (!dnf_repo_set_keyfile_data(repo, error))
+    if (!dnf_repo_set_keyfile_data(repo, TRUE, error))
         goto out;
 
     /* if nothing specified then use cachedir */
@@ -2322,8 +2312,10 @@ dnf_repo_download_packages(DnfRepo *repo,
 
     ret = TRUE;
 out:
-    lr_handle_setopt(priv->repo_handle, NULL, LRO_PROGRESSCB, NULL);
-    lr_handle_setopt(priv->repo_handle, NULL, LRO_PROGRESSDATA, 0xdeadbeef);
+    if (!lr_handle_setopt(priv->repo_handle, NULL, LRO_PROGRESSCB, NULL))
+            g_debug("Failed to reset LRO_PROGRESSCB to NULL");
+    if (!lr_handle_setopt(priv->repo_handle, NULL, LRO_PROGRESSDATA, 0xdeadbeef))
+            g_debug("Failed to set LRO_PROGRESSDATA to 0xdeadbeef");
     g_free(global_data.last_mirror_failure_message);
     g_free(global_data.last_mirror_url);
     g_slist_free_full(package_targets, (GDestroyNotify)lr_packagetarget_free);
