@@ -80,7 +80,6 @@ extern "C" {
 #include "module/ModulePackage.hpp"
 #include "repo/Repo-private.hpp"
 #include "repo/solvable/DependencyContainer.hpp"
-#include "utils/crypto/sha1.hpp"
 #include "utils/File.hpp"
 #include "utils/utils.hpp"
 #include "log.hpp"
@@ -225,17 +224,39 @@ dnf_sack_new(void)
     return DNF_SACK(g_object_new(DNF_TYPE_SACK, NULL));
 }
 
-static int
-can_use_repomd_cache(FILE *fp_solv, unsigned char cs_repomd[CHKSUM_BYTES])
-{
-    unsigned char cs_cache[CHKSUM_BYTES];
+// Try to load cached solv file into repo otherwise return FALSE
+static gboolean
+try_to_use_cached_solvfile(const char *path, Repo *repo, int flags, const unsigned char *checksum, GError **err){
+    FILE *fp_cache = fopen(path, "r");
+    if (!fp_cache) {
+        // Missing cache files (ENOENT) are not an error and can even be expected in some cases
+        // (such as when repo doesn't have updateinfo/prestodelta metadata).
+        // Use g_debug in order not to pollute the log by default with such entries.
+        if (errno == ENOENT) {
+            g_debug("Failed to open solvfile cache: %s: %s", path, strerror(errno));
+        } else {
+            g_warning("Failed to open solvfile cache: %s: %s", path, strerror(errno));
+        }
+        return FALSE;
+    }
+    std::unique_ptr<SolvUserdata> solv_userdata = solv_userdata_read(fp_cache);
+    gboolean ret = TRUE;
+    if (solv_userdata && solv_userdata_verify(solv_userdata.get(), checksum)) {
+        // after reading the header rewind to the begining
+        fseek(fp_cache, 0, SEEK_SET);
+        if (repo_add_solv(repo, fp_cache, flags)) {
+            g_set_error (err,
+                         DNF_ERROR,
+                         DNF_ERROR_INTERNAL_ERROR,
+                         _("repo_add_solv() has failed."));
+            ret = FALSE;
+        }
+    } else {
+        ret = FALSE;
+    }
 
-    if (fp_solv &&
-        !checksum_read(cs_cache, fp_solv) &&
-        !checksum_cmp(cs_cache, cs_repomd))
-        return 1;
-
-    return 0;
+    fclose(fp_cache);
+    return ret;
 }
 
 void
@@ -371,10 +392,35 @@ load_ext(DnfSack *sack, HyRepo hrepo, _hy_repo_repodata which_repodata,
     auto repoImpl = libdnf::repoGetImpl(hrepo);
     Repo *repo = repoImpl->libsolvRepo;
     const char *name = repo->name;
-    auto fn = hrepo->getMetadataPath(which_filename);
     FILE *fp;
     gboolean done = FALSE;
 
+    char *fn_cache =  dnf_sack_give_cache_fn(sack, name, suffix);
+    assert(libdnf::repoGetImpl(hrepo)->checksum);
+
+    int flags = 0;
+    /* the updateinfo is not a real extension */
+    if (which_repodata != _HY_REPODATA_UPDATEINFO)
+        flags |= REPO_EXTEND_SOLVABLES;
+    /* do not pollute the main pool with directory component ids */
+    if (which_repodata == _HY_REPODATA_FILENAMES || which_repodata == _HY_REPODATA_OTHER)
+        flags |= REPO_LOCALPOOL;
+    if (try_to_use_cached_solvfile(fn_cache, repo, flags, libdnf::repoGetImpl(hrepo)->checksum, error)) {
+        g_debug("%s: using cache file: %s", __func__, fn_cache);
+        done = TRUE;
+        repo_update_state(hrepo, which_repodata, _HY_LOADED_CACHE);
+        repo_set_repodata(hrepo, which_repodata, repo->nrepodata - 1);
+    }
+    if (error && *error) {
+        g_prefix_error(error, _("Loading extension cache %s (%d) failed: "), fn_cache, which_repodata);
+        return FALSE;
+    }
+
+    g_free(fn_cache);
+    if (done)
+        return TRUE;
+
+    auto fn = hrepo->getMetadataPath(which_filename);
     /* nothing set */
     if (fn.empty()) {
         g_set_error (error,
@@ -385,37 +431,6 @@ load_ext(DnfSack *sack, HyRepo hrepo, _hy_repo_repodata which_repodata,
         return FALSE;
     }
 
-    char *fn_cache =  dnf_sack_give_cache_fn(sack, name, suffix);
-    fp = fopen(fn_cache, "r");
-    assert(libdnf::repoGetImpl(hrepo)->checksum);
-    if (can_use_repomd_cache(fp, libdnf::repoGetImpl(hrepo)->checksum)) {
-        int flags = 0;
-        /* the updateinfo is not a real extension */
-        if (which_repodata != _HY_REPODATA_UPDATEINFO)
-            flags |= REPO_EXTEND_SOLVABLES;
-        /* do not pollute the main pool with directory component ids */
-        if (which_repodata == _HY_REPODATA_FILENAMES || which_repodata == _HY_REPODATA_OTHER)
-            flags |= REPO_LOCALPOOL;
-        done = TRUE;
-        g_debug("%s: using cache file: %s", __func__, fn_cache);
-        ret = repo_add_solv(repo, fp, flags);
-        if (ret) {
-            g_set_error_literal (error,
-                                 DNF_ERROR,
-                                 DNF_ERROR_INTERNAL_ERROR,
-                                 _("failed to add solv"));
-            return FALSE;
-        } else {
-            repo_update_state(hrepo, which_repodata, _HY_LOADED_CACHE);
-            repo_set_repodata(hrepo, which_repodata, repo->nrepodata - 1);
-        }
-    }
-    g_free(fn_cache);
-    if (fp)
-        fclose(fp);
-    if (done)
-        return TRUE;
-
     fp = solv_xfopen(fn.c_str(), "r");
     if (fp == NULL) {
         g_set_error (error,
@@ -514,35 +529,53 @@ write_main(DnfSack *sack, HyRepo hrepo, int switchtosolv, GError **error)
                         strerror(errno));
             goto done;
         }
-        rc = repo_write(repo, fp);
-        rc |= checksum_write(repoImpl->checksum, fp);
-        rc |= fclose(fp);
+
+        SolvUserdata solv_userdata;
+        if (solv_userdata_fill(&solv_userdata, repoImpl->checksum, error)) {
+            ret = FALSE;
+            fclose(fp);
+            goto done;
+        }
+
+        Repowriter *writer = repowriter_create(repo);
+        repowriter_set_userdata(writer, &solv_userdata, solv_userdata_size);
+        rc = repowriter_write(writer, fp);
+        repowriter_free(writer);
         if (rc) {
+            ret = FALSE;
+            fclose(fp);
+            g_set_error(error,
+                       DNF_ERROR,
+                       DNF_ERROR_INTERNAL_ERROR,
+                       _("While writing primary cache %s repowriter write failed: %i, error: %s"),
+                       tmp_fn_templ, rc, pool_errstr(repo->pool));
+            goto done;
+        }
+
+        if (fclose(fp)) {
             ret = FALSE;
             g_set_error (error,
                         DNF_ERROR,
                         DNF_ERROR_FILE_INVALID,
-                        _("write_main() failed writing data: %i"), rc);
+                        _("Failed closing tmp file %s: %s"),
+                        tmp_fn_templ, strerror(errno));
             goto done;
         }
     }
     if (switchtosolv && repo_is_one_piece(repo)) {
+        repo_empty(repo, 1);
         /* switch over to written solv file activate paging */
-        FILE *fp = fopen(tmp_fn_templ, "r");
-        if (fp) {
-            repo_empty(repo, 1);
-            rc = repo_add_solv(repo, fp, 0);
-            fclose(fp);
-            if (rc) {
-                /* this is pretty fatal */
-                ret = FALSE;
-                g_set_error_literal (error,
-                                     DNF_ERROR,
-                                     DNF_ERROR_FILE_INVALID,
-                                     _("write_main() failed to re-load "
-                                       "written solv file"));
-                goto done;
-            }
+        gboolean loaded = try_to_use_cached_solvfile(tmp_fn_templ, repo, 0, repoImpl->checksum, error);
+        if (error && *error) {
+            g_prefix_error(error, _("Failed to use newly written primary cache: %s: "), tmp_fn_templ);
+            ret = FALSE;
+            goto done;
+        }
+        if (!loaded) {
+            g_set_error(error, DNF_ERROR, DNF_ERROR_INTERNAL_ERROR,
+                        _("Failed to use newly written primary cache: %s"), tmp_fn_templ);
+            ret = FALSE;
+            goto done;
         }
     }
 
@@ -569,20 +602,6 @@ write_ext_updateinfo_filter(Repo *repo, Repokey *key, void *kfdata)
     return repo_write_stdkeyfilter(repo, key, 0);
 }
 
-static int
-write_ext_updateinfo(HyRepo hrepo, Repodata *data, FILE *fp)
-{
-    auto repoImpl = libdnf::repoGetImpl(hrepo);
-    Repo *repo = repoImpl->libsolvRepo;
-    int oldstart = repo->start;
-    repo->start = repoImpl->main_end;
-    repo->nsolvables -= repoImpl->main_nsolvables;
-    int res = repo_write_filtered(repo, fp, write_ext_updateinfo_filter, data, 0);
-    repo->start = oldstart;
-    repo->nsolvables += repoImpl->main_nsolvables;
-    return res;
-}
-
 static gboolean
 write_ext(DnfSack *sack, HyRepo hrepo, _hy_repo_repodata which_repodata,
           const char *suffix, GError **error)
@@ -611,37 +630,78 @@ write_ext(DnfSack *sack, HyRepo hrepo, _hy_repo_repodata which_repodata,
         FILE *fp = fdopen(tmp_fd, "w+");
 
         g_debug("%s: storing %s to: %s", __func__, repo->name, tmp_fn_templ);
-        if (which_repodata != _HY_REPODATA_UPDATEINFO)
-            ret |= repodata_write(data, fp);
-        else
-            ret |= write_ext_updateinfo(hrepo, data, fp);
-        ret |= checksum_write(repoImpl->checksum, fp);
-        ret |= fclose(fp);
+
+        SolvUserdata solv_userdata;
+        if (solv_userdata_fill(&solv_userdata, repoImpl->checksum, error)) {
+            fclose(fp);
+            success = FALSE;
+            goto done;
+        }
+
+        Repowriter *writer = repowriter_create(repo);
+        repowriter_set_userdata(writer, &solv_userdata, solv_userdata_size);
+        if (which_repodata != _HY_REPODATA_UPDATEINFO) {
+            repowriter_set_repodatarange(writer, data->repodataid, data->repodataid + 1);
+            repowriter_set_flags(writer, REPOWRITER_NO_STORAGE_SOLVABLE);
+            ret = repowriter_write(writer, fp);
+        } else {
+            // write only updateinfo repodata
+            int oldstart = repo->start;
+            repo->start = repoImpl->main_end;
+            repo->nsolvables -= repoImpl->main_nsolvables;
+            repowriter_set_flags(writer, REPOWRITER_LEGACY);
+            repowriter_set_keyfilter(writer, write_ext_updateinfo_filter, data);
+            repowriter_set_keyqueue(writer, 0);
+            ret = repowriter_write(writer, fp);
+            repo->start = oldstart;
+            repo->nsolvables += repoImpl->main_nsolvables;
+        }
+        repowriter_free(writer);
         if (ret) {
+            success = FALSE;
+            fclose(fp);
+            g_set_error (error,
+                        DNF_ERROR,
+                        DNF_ERROR_INTERNAL_ERROR,
+                        _("While writing extension cache %s (%d): repowriter write failed: %i, error: %s"),
+                        tmp_fn_templ, which_repodata, ret, pool_errstr(repo->pool));
+            goto done;
+        }
+
+        if (fclose(fp)) {
             success = FALSE;
             g_set_error (error,
                         DNF_ERROR,
-                        DNF_ERROR_FAILED,
-                        _("write_ext(%1$d) has failed: %2$d"),
-                        which_repodata, ret);
+                        DNF_ERROR_FILE_INVALID,
+                        _("While writing extension cache (%d): cannot close temporary file: %s"),
+                        which_repodata, tmp_fn_templ);
             goto done;
         }
     }
 
     if (repo_is_one_piece(repo) && which_repodata != _HY_REPODATA_UPDATEINFO) {
         /* switch over to written solv file activate paging */
-        FILE *fp = fopen(tmp_fn_templ, "r");
-        if (fp) {
-            int flags = REPO_USE_LOADING | REPO_EXTEND_SOLVABLES;
-            /* do not pollute the main pool with directory component ids */
-            if (which_repodata == _HY_REPODATA_FILENAMES || which_repodata == _HY_REPODATA_OTHER)
-                flags |= REPO_LOCALPOOL;
-            repodata_extend_block(data, repo->start, repo->end - repo->start);
-            data->state = REPODATA_LOADING;
-            repo_add_solv(repo, fp, flags);
-            data->state = REPODATA_AVAILABLE;
-            fclose(fp);
+        int flags = REPO_USE_LOADING | REPO_EXTEND_SOLVABLES;
+        /* do not pollute the main pool with directory component ids */
+        if (which_repodata == _HY_REPODATA_FILENAMES || which_repodata == _HY_REPODATA_OTHER)
+            flags |= REPO_LOCALPOOL;
+        repodata_extend_block(data, repo->start, repo->end - repo->start);
+        data->state = REPODATA_LOADING;
+        int loaded = try_to_use_cached_solvfile(tmp_fn_templ, repo, flags, repoImpl->checksum, error);
+        if (error && *error) {
+            g_prefix_error(error, _("Failed to use newly written extension cache: %s (%d): "),
+                           tmp_fn_templ, which_repodata);
+            success = FALSE;
+            goto done;
         }
+        if (!loaded) {
+            g_set_error(error, DNF_ERROR, DNF_ERROR_INTERNAL_ERROR,
+                        _("Failed to use newly written extension cache: %s (%d)"), tmp_fn_templ, which_repodata);
+            success = FALSE;
+            goto done;
+        }
+
+        data->state = REPODATA_AVAILABLE;
     }
 
     if (!mv(tmp_fn_templ, fn, error)) {
@@ -672,7 +732,7 @@ load_yum_repo(DnfSack *sack, HyRepo hrepo, GError **error)
 
     FILE *fp_primary = NULL;
     FILE *fp_repomd = NULL;
-    FILE *fp_cache = fopen(fn_cache, "r");
+
     if (!fn_repomd) {
         g_set_error (error,
                      DNF_ERROR,
@@ -693,18 +753,17 @@ load_yum_repo(DnfSack *sack, HyRepo hrepo, GError **error)
     }
     checksum_fp(repoImpl->checksum, fp_repomd);
 
-    if (can_use_repomd_cache(fp_cache, repoImpl->checksum)) {
+    if (try_to_use_cached_solvfile(fn_cache, repo, 0, repoImpl->checksum, error)) {
         const char *chksum = pool_checksum_str(pool, repoImpl->checksum);
         g_debug("using cached %s (0x%s)", name, chksum);
-        if (repo_add_solv(repo, fp_cache, 0)) {
-            g_set_error (error,
-                         DNF_ERROR,
-                         DNF_ERROR_INTERNAL_ERROR,
-                         _("repo_add_solv() has failed."));
-            retval = FALSE;
-            goto out;
-        }
         repoImpl->state_main = _HY_LOADED_CACHE;
+        goto out;
+    }
+
+    if (error && *error) {
+        g_prefix_error(error, _("While loading repository failed to use %s: "), fn_cache);
+        retval = FALSE;
+        goto out;
     } else {
         auto primary = hrepo->getMetadataPath(MD_TYPE_PRIMARY);
         if (primary.empty()) {
@@ -718,23 +777,40 @@ load_yum_repo(DnfSack *sack, HyRepo hrepo, GError **error)
             goto out;
         }
         fp_primary = solv_xfopen(primary.c_str(), "r");
-        assert(fp_primary);
+        if (fp_primary == 0) {
+            g_set_error(error,
+                        DNF_ERROR,
+                        DNF_ERROR_INTERNAL_ERROR,
+                        _("Opening repository primary data has failed: %s"),
+                        strerror(errno));
+            retval = FALSE;
+            goto out;
+        }
 
-        g_debug("fetching %s", name);
-        if (repo_add_repomdxml(repo, fp_repomd, 0) || \
-            repo_add_rpmmd(repo, fp_primary, 0, 0)) {
+        g_debug("Loading repomd: %s", fn_repomd);
+        if (repo_add_repomdxml(repo, fp_repomd, 0)) {
             g_set_error (error,
                          DNF_ERROR,
                          DNF_ERROR_INTERNAL_ERROR,
-                         _("repo_add_repomdxml/rpmmd() has failed."));
+                         _("Loading repomd has failed: %s"),
+                         pool_errstr(repo->pool));
+            retval = FALSE;
+            goto out;
+        }
+
+        g_debug("Loading primary: %s", primary.c_str());
+        if (repo_add_rpmmd(repo, fp_primary, 0, 0)) {
+            g_set_error (error,
+                         DNF_ERROR,
+                         DNF_ERROR_INTERNAL_ERROR,
+                         _("Loading primary has failed: %s"),
+                         pool_errstr(repo->pool));
             retval = FALSE;
             goto out;
         }
         repoImpl->state_main = _HY_LOADED_FETCH;
     }
 out:
-    if (fp_cache)
-        fclose(fp_cache);
     if (fp_repomd)
         fclose(fp_repomd);
     if (fp_primary)
@@ -2297,7 +2373,6 @@ void readModuleMetadataFromRepo(DnfSack * sack, libdnf::ModulePackageContainer *
     modulePackages->add(sack);
     modulePackages->loadFailSafeData();
     if (!modulePackages->empty()) {
-        modulePackages->createConflictsBetweenStreams();
         // TODO remove hard-coded path
         try {
             std::vector<std::string> paths{"/etc/os-release", "/usr/lib/os-release"};
@@ -2309,47 +2384,107 @@ void readModuleMetadataFromRepo(DnfSack * sack, libdnf::ModulePackageContainer *
     }
 }
 
-static std::tuple<std::vector<std::string>, std::vector<std::string>>
-collectNevraForInclusionExclusion(libdnf::ModulePackageContainer &modulePackageContainer)
+/// @return std::map<name:stream.arch, std::vector<std::string>>
+static std::map<std::string, std::vector<std::string>> getDemodularizedRpms(
+    libdnf::ModulePackageContainer & moduleContainer, const std::vector<libdnf::ModulePackage *> & allPackages)
 {
+    std::map<std::string, std::vector<std::string>> ret;
+    auto latest = moduleContainer.getLatestModules(allPackages, true);
+    for (auto modulePackage : latest) {
+        auto demodularized = modulePackage->getDemodularizedRpms();
+        if (demodularized.empty()) {
+            continue;
+        }
+        std::string packageID{modulePackage->getNameStream()};
+        packageID.append(".");
+        packageID.append(modulePackage->getArch());
+        auto & data = ret[packageID];
+        data.insert(data.end(), demodularized.begin(), demodularized.end());
+    }
+    return ret;
+}
+
+/// Return <includeNEVRAs>, <excludeNEVRAs>, <names>, <srcNames>, <name dependency container>
+static std::tuple<std::vector<std::string>, std::vector<std::string>, std::vector<std::string>, std::vector<std::string>, libdnf::DependencyContainer>
+collectNevraForInclusionExclusion(DnfSack *sack, libdnf::ModulePackageContainer &modulePackageContainer)
+{
+    auto allPackages = modulePackageContainer.getModulePackages();
+    auto demodularizedNames = getDemodularizedRpms(modulePackageContainer, allPackages);
+
     std::vector<std::string> includeNEVRAs;
     std::vector<std::string> excludeNEVRAs;
+    std::vector<std::string> names;
+    std::vector<std::string> srcNames;
+    libdnf::DependencyContainer nameDependencies{sack};
+    libdnf::Nevra nevra;
     // TODO: turn into std::vector<const char *> to prevent unnecessary conversion?
-    for (const auto &module : modulePackageContainer.getModulePackages()) {
+    for (const auto & module : allPackages) {
         auto artifacts = module->getArtifacts();
         // TODO use Goal::listInstalls() to not requires filtering out Platform
         if (modulePackageContainer.isModuleActive(module->getId())) {
+            std::string packageID{module->getNameStream()};
+            packageID.append(".");
+            packageID.append(module->getArch());
+            auto it = demodularizedNames.find(packageID);
+            if (it == demodularizedNames.end()) {
+                for (const auto &rpm : artifacts) {
+                    if (nevra.parse(rpm.c_str(), HY_FORM_NEVRA)) {
+                        auto arch = nevra.getArch();
+                        // source packages do not provide anything and must not cause excluding binary packages
+                        if (arch == "src" || arch == "nosrc") {
+                            srcNames.push_back(nevra.getName());
+                        } else {
+                            names.push_back(nevra.getName());
+                            nameDependencies.addReldep(nevra.getName().c_str());
+                        }
+                    }
+                }
+            } else {
+                for (const auto &rpm : artifacts) {
+                    if (nevra.parse(rpm.c_str(), HY_FORM_NEVRA)) {
+                        bool found = false;
+                        for ( auto & demodularized : it->second) {
+                            if (nevra.getName() == demodularized) {
+                                found = true;
+                                break;
+                            }
+                        }
+                        if (found) {
+                            continue;
+                        }
+                        auto arch = nevra.getArch();
+                        // source packages do not provide anything and must not cause excluding binary packages
+                        if (arch == "src" || arch == "nosrc") {
+                            srcNames.push_back(nevra.getName());
+                        } else {
+                            names.push_back(nevra.getName());
+                            nameDependencies.addReldep(nevra.getName().c_str());
+                        }
+                    }
+                }
+            }
             copy(std::begin(artifacts), std::end(artifacts), std::back_inserter(includeNEVRAs));
         } else {
             copy(std::begin(artifacts), std::end(artifacts), std::back_inserter(excludeNEVRAs));
         }
     }
 
-    return std::tuple<std::vector<std::string>,
-    std::vector<std::string>>{includeNEVRAs, excludeNEVRAs};
+    return std::make_tuple(std::move(includeNEVRAs), std::move(excludeNEVRAs), std::move(names), std::move(srcNames),
+                           std::move(nameDependencies));
 }
 
-static void
-setModuleExcludes(DnfSack *sack, const char ** hotfixRepos,
-    const std::vector<std::string> &includeNEVRAs, const std::vector<std::string> &excludeNEVRAs)
+void
+setModuleExcludes(DnfSack * sack, const char ** hotfixRepos, libdnf::ModulePackageContainer & modulePackageContainer)
 {
     dnf_sack_set_module_excludes(sack, nullptr);
-    std::vector<std::string> names;
-    std::vector<std::string> srcNames;
-    libdnf::DependencyContainer nameDependencies{sack};
-    libdnf::Nevra nevra;
-    for (const auto &rpm : includeNEVRAs) {
-        if (nevra.parse(rpm.c_str(), HY_FORM_NEVRA)) {
-            auto arch = nevra.getArch();
-            // source packages do not provide anything and must not cause excluding binary packages
-            if (arch == "src" || arch == "nosrc") {
-                srcNames.push_back(nevra.getName());
-            } else {
-                names.push_back(nevra.getName());
-                nameDependencies.addReldep(nevra.getName().c_str());
-            }
-        }
-    }
+
+    auto data = collectNevraForInclusionExclusion(sack, modulePackageContainer);
+
+    auto & includeNEVRAs = std::get<0>(data);
+    auto & excludeNEVRAs = std::get<1>(data);
+    auto & names = std::get<2>(data);
+    auto & srcNames = std::get<3>(data);
+    auto & nameDependencies = std::get<4>(data);
 
     std::vector<const char *> namesCString(names.size() + 1);
     std::vector<const char *> srcNamesCString(srcNames.size() + 1);
@@ -2366,7 +2501,9 @@ setModuleExcludes(DnfSack *sack, const char ** hotfixRepos,
     libdnf::Query keepPackages{sack};
     const char *keepRepo[] = {HY_CMDLINE_REPO_NAME, HY_SYSTEM_REPO_NAME, nullptr};
     keepPackages.addFilter(HY_PKG_REPONAME, HY_NEQ, keepRepo);
-    keepPackages.addFilter(HY_PKG_REPONAME, HY_NEQ, hotfixRepos);
+    if (hotfixRepos != nullptr) {
+        keepPackages.addFilter(HY_PKG_REPONAME, HY_NEQ, hotfixRepos);
+    }
 
     libdnf::Query includeQuery{sack};
     libdnf::Query excludeQuery{keepPackages};
@@ -2431,12 +2568,12 @@ void dnf_sack_filter_modules(DnfSack *sack, GPtrArray *repos, const char *instal
     }
     hotfixRepos.push_back(nullptr);
     dnf_sack_filter_modules_v2(sack, nullptr, hotfixRepos.data(), install_root, platformModule,
-                               false, false);
+                               false, false, false);
 }
 
 std::pair<std::vector<std::vector<std::string>>, libdnf::ModulePackageContainer::ModuleErrorType> dnf_sack_filter_modules_v2(
     DnfSack * sack, DnfModulePackageContainer * moduleContainer, const char ** hotfixRepos,
-    const char * install_root, const char * platformModule, bool updateOnly, bool debugSolver)
+    const char * install_root, const char * platformModule, bool updateOnly, bool debugSolver, bool applyObsoletes)
 {
     if (!updateOnly) {
         if (!install_root) {
@@ -2465,48 +2602,12 @@ std::pair<std::vector<std::vector<std::string>>, libdnf::ModulePackageContainer:
     if (!moduleContainer) {
         throw std::runtime_error("ModuleContainer not provided");
     }
-    auto ret = moduleContainer->resolveActiveModulePackages(debugSolver);
-    auto nevraTuple = collectNevraForInclusionExclusion(*moduleContainer);
 
-    setModuleExcludes(sack, hotfixRepos, std::get<0>(nevraTuple), std::get<1>(nevraTuple));
+    if (applyObsoletes) {
+        moduleContainer->applyObsoletes();
+    }
+    auto ret = moduleContainer->resolveActiveModulePackages(debugSolver);
+
+    setModuleExcludes(sack, hotfixRepos, *moduleContainer);
     return ret;
 }
-
-std::string dnf_sack_get_rpmdb_version(DnfSack *sack) {
-    // collect all sha1hdr checksums
-    // they are sufficiently unique IDs that represent installed RPMs
-    std::vector<std::string> checksums;
-
-    // iterate all @System repo RPMs (rpmdb records)
-    libdnf::Query query{sack, libdnf::Query::ExcludeFlags::IGNORE_EXCLUDES};
-    query.installed();
-
-    auto pset = query.getResultPset();
-    Id id = -1;
-    while(true) {
-        id = pset->next(id);
-        if (id == -1) {
-            break;
-        }
-        DnfPackage *pkg = dnf_package_new(sack, id);
-        // store pkgid (equals to sha1hdr)
-        checksums.push_back(libdnf::string::fromCstring(dnf_package_get_pkgid(pkg)));
-        g_object_unref(pkg);
-    }
-
-    // sort checksums to compute the output checksum always the same
-    std::sort(checksums.begin(), checksums.end());
-
-    SHA1Hash h;
-    for (auto & checksum : checksums) {
-        h.update(checksum.c_str());
-    }
-
-    // build <count>:<hash> output
-    std::ostringstream result;
-    result << checksums.size();
-    result << ":";
-    result << h.hexdigest();
-
-    return result.str();
-}
