@@ -22,7 +22,11 @@
 #include <errno.h>
 #include <dirent.h>
 #include <fcntl.h>
+#ifdef __APPLE__
+#include <limits.h>
+#else
 #include <linux/limits.h>
+#endif
 #include <pwd.h>
 #include <unistd.h>
 #include <stdio.h>
@@ -39,6 +43,7 @@ extern "C" {
 #include <solv/evr.h>
 #include <solv/solver.h>
 #include <solv/solverdebug.h>
+#include <solv/repo_solv.h>
 #include <solv/util.h>
 #include <solv/pool_parserpmrichdep.h>
 }
@@ -137,17 +142,6 @@ checksum_fp(unsigned char *out, FILE *fp)
     return 0;
 }
 
-/* calls rewind(fp) before returning */
-int
-checksum_read(unsigned char *csout, FILE *fp)
-{
-    if (fseek(fp, -32, SEEK_END) ||
-        fread(csout, CHKSUM_BYTES, 1, fp) != 1)
-        return 1;
-    rewind(fp);
-    return 0;
-}
-
 /* does not move the fp position */
 int
 checksum_stat(unsigned char *out, FILE *fp)
@@ -169,15 +163,107 @@ checksum_stat(unsigned char *out, FILE *fp)
     return 0;
 }
 
-/* moves fp to the end of file */
-int checksum_write(const unsigned char *cs, FILE *fp)
+static std::array<char, solv_userdata_solv_toolversion_size>
+get_padded_solv_toolversion()
 {
-    if (fseek(fp, 0, SEEK_END) ||
-        fwrite(cs, CHKSUM_BYTES, 1, fp) != 1)
+    std::array<char, solv_userdata_solv_toolversion_size> padded_solv_toolversion{};
+    std::string solv_ver_str{solv_toolversion};
+    std::copy(solv_ver_str.rbegin(), solv_ver_str.rend(), padded_solv_toolversion.rbegin());
+
+    return padded_solv_toolversion;
+}
+
+int
+solv_userdata_fill(SolvUserdata *solv_userdata, const unsigned char *checksum, GError** error)
+{
+    if (strlen(solv_toolversion) > solv_userdata_solv_toolversion_size) {
+        g_set_error(error, DNF_ERROR, DNF_ERROR_INTERNAL_ERROR,
+                    _("Libsolv's solv_toolversion is: %zu long but we expect max of: %zu"),
+                    strlen(solv_toolversion), solv_userdata_solv_toolversion_size);
         return 1;
+    }
+
+    // copy dnf solv file magic
+    memcpy(solv_userdata->dnf_magic, solv_userdata_magic.data(), solv_userdata_magic.size());
+
+    // copy dnf solv file version
+    memcpy(solv_userdata->dnf_version, solv_userdata_dnf_version.data(), solv_userdata_dnf_version.size());
+
+    // copy libsolv solv file version
+    memcpy(solv_userdata->libsolv_version, get_padded_solv_toolversion().data(), solv_userdata_solv_toolversion_size);
+
+    // copy checksum
+    memcpy(solv_userdata->checksum, checksum, CHKSUM_BYTES);
+
     return 0;
 }
 
+
+std::unique_ptr<SolvUserdata>
+solv_userdata_read(FILE *fp)
+{
+    unsigned char *dnf_solvfile_userdata_read = NULL;
+    int dnf_solvfile_userdata_len_read;
+    if (!fp) {
+        return nullptr;
+    }
+
+    int ret_code = solv_read_userdata(fp, &dnf_solvfile_userdata_read, &dnf_solvfile_userdata_len_read);
+    // The userdata layout has to match our struct exactly so we can just cast the memory
+    // allocated by libsolv
+    std::unique_ptr<SolvUserdata> uniq_userdata(reinterpret_cast<SolvUserdata *>(dnf_solvfile_userdata_read));
+    if(ret_code) {
+        g_warning("Failed to read solv userdata: solv_read_userdata returned: %i", ret_code);
+        return nullptr;
+    }
+
+    if (dnf_solvfile_userdata_len_read != solv_userdata_size) {
+        g_warning("Solv userdata length mismatch, read: %i vs expected: %i",
+                  dnf_solvfile_userdata_len_read, solv_userdata_size);
+        return nullptr;
+    }
+
+    return uniq_userdata;
+}
+
+gboolean
+solv_userdata_verify(const SolvUserdata *solv_userdata, const unsigned char *checksum)
+{
+    // check dnf solvfile magic bytes
+    if (memcmp(solv_userdata->dnf_magic, solv_userdata_magic.data(), solv_userdata_magic.size()) != 0) {
+        // This is not dnf header do not read after it
+        g_warning("magic bytes don't match, read: %s vs. dnf solvfile magic: %s",
+                  solv_userdata->dnf_magic, solv_userdata_magic.data());
+        return FALSE;
+    }
+
+    // check dnf solvfile version
+    if (memcmp(solv_userdata->dnf_version, solv_userdata_dnf_version.data(), solv_userdata_dnf_version.size()) != 0) {
+        // Mismatching dnf solvfile version -> we need to regenerate
+        g_warning("dnf solvfile version doesn't match, read: %s vs. dnf solvfile version: %s",
+                  solv_userdata->dnf_version, solv_userdata_dnf_version.data());
+        return FALSE;
+    }
+
+    // check libsolv solvfile version
+    if (memcmp(solv_userdata->libsolv_version, get_padded_solv_toolversion().data(), solv_userdata_solv_toolversion_size) != 0) {
+        // Mismatching libsolv solvfile version -> we need to regenerate
+        g_warning("libsolv solvfile version doesn't match, read: %s vs. libsolv version: %s",
+                  solv_userdata->libsolv_version, solv_toolversion);
+        return FALSE;
+    }
+
+    // check solvfile checksum
+    if (checksum_cmp(solv_userdata->checksum, checksum)) {
+        // Mismatching solvfile checksum -> we need to regenerate
+        g_debug("solvfile checksum doesn't match, read: %s vs. repomd checksum: %s",
+                solv_userdata->checksum, checksum);
+        return FALSE;
+    }
+
+    return TRUE;
+}
+
 int
 checksum_type2length(int type)
 {
