@@ -197,6 +197,42 @@ NamePrioritySolvableKey(const Solvable * first, const Solvable * second)
     return first->repo->priority > second->repo->priority;
 }
 
+static bool
+NameArchPrioritySolvableKey(const Solvable * first, const Solvable * second)
+{
+    if (first->name != second->name)
+        return first->name < second->name;
+    if (first->arch != second->arch) {
+        return first->arch < second->arch;
+    }
+    return first->repo->priority > second->repo->priority;
+}
+
+struct NameArchEVRComparator {
+   NameArchEVRComparator(Pool * pool) : pool(pool) {};
+   bool operator()(const Solvable * first, const Solvable * second) {
+       if (first->name != second->name) {
+          return first->name < second->name;
+       }
+       if (first->arch != second->arch) {
+          return first->arch < second->arch;
+       }
+       return pool_evrcmp(pool, first->evr, second->evr, EVRCMP_COMPARE) < 0;
+   }
+   bool operator()(const Solvable * solvable, const AdvisoryPkg & pkg) {
+       if (pkg.getName() != solvable->name) {
+          return pkg.getName() > solvable->name;
+        }
+        if (pkg.getArch() != solvable->arch) {
+            return pkg.getArch() > solvable->arch;
+        }
+        return pool_evrcmp(pool, pkg.getEVR(), solvable->evr, EVRCMP_COMPARE) > 0;
+   }
+
+   Pool * pool;
+};
+
+
 static bool
 match_type_num(int keyname) {
     switch (keyname) {
@@ -606,6 +642,14 @@ advisoryPkgCompareSolvableNameArch(const AdvisoryPkg &first, const Solvable &s)
     return first.getArch() < s.arch;
 }
 
+static bool
+SolvableCompareAdvisoryPkgNameArch(const Solvable * s, const AdvisoryPkg & first)
+{
+    if (first.getName() != s->name)
+        return first.getName() > s->name;
+    return first.getArch() > s->arch;
+}
+
 static char *
 copyFilterChar(const char * match, int keyname)
 {
@@ -1769,7 +1813,9 @@ Query::Impl::filterLocation(const Filter & f, Map *m)
 }
 
 /**
-* @brief Reduce query to security filters. It reflect following compare types: HY_EQ, HY_GT, HY_LT.
+* @brief Reduce query to security filters. It reflect following compare types: HY_EQ, HY_GT, HY_LT. Additionally it is
+* possible to use HY_EQG. HY_EQG can be combine with HY_UPGRADE or HY_GT. HY_UPGRADE skips advisory that arr already
+* resolved by installed packages. It also select results according priority (important for upgrade-minimal).
 *
 * @param f: Filter that should be applied on advisories
 * @param m: Map of query results complying the filter
@@ -1816,9 +1862,7 @@ Query::Impl::filterAdvisory(const Filter & f, Map *m, int keyname)
                     eq = false;
             }
             if (eq) {
-                if (isAdvisoryApplicable(advisory, sack)) {
-                    advisory.getPackages(pkgs, false);
-                }
+                advisory.getApplicablePackages(pkgs, false);
                 break;
             }
         }
@@ -1827,32 +1871,139 @@ Query::Impl::filterAdvisory(const Filter & f, Map *m, int keyname)
     dataiterator_free(&di);
     std::sort(pkgs.begin(), pkgs.end(), advisoryPkgSort);
 
-    // convert nevras (from DnfAdvisoryPkg) to pool ids
-    Id id = -1;
     int cmp_type = f.getCmpType();
-    while (true) {
-        if (pkgs.size() == 0)
-            break;
-        id = resultPset->next(id);
-        if (id == -1)
-            break;
-        Solvable* s = pool_id2solvable(pool, id);
-        if (cmp_type == HY_EQ) {
-            auto low = std::lower_bound(pkgs.begin(), pkgs.end(), *s, advisoryPkgCompareSolvable);
-            if (low != pkgs.end() && low->nevraEQ(s)) {
-                MAPSET(m, id);
+
+    if (cmp_type & HY_EQG) {
+        std::vector<Solvable *> candidates;
+        std::vector<Solvable *> installed_solvables;
+
+        if (cmp_type & HY_UPGRADE) {
+            // When doing HY_UPGRADE consider only candidate pkgs that have matching Name and Arch with:
+            // * some already installed pkg (in other words: some other version of the pkg is already installed)
+            // or
+            // * with pkg that obsoletes some already installed (or to be installed in this transaction) pkg
+            // Otherwise a pkg with different Arch than installed can end up in upgrade set which is wrong.
+            // It can result in dependency issues, reported as: RhBug:2088149.
+
+            Query installed(sack, ExcludeFlags::IGNORE_EXCLUDES);
+            installed.installed();
+            installed.addFilter(HY_PKG_LATEST_PER_ARCH, HY_EQ, 1);
+            installed.apply();
+            Id installed_id = -1;
+            while ((installed_id = installed.pImpl->result->next(installed_id)) != -1) {
+                installed_solvables.push_back(pool_id2solvable(pool, installed_id));
             }
+            std::sort(installed_solvables.begin(), installed_solvables.end(), NameArchSolvableComparator);
+
+            Query obsoletes(sack, ExcludeFlags::IGNORE_EXCLUDES);
+            obsoletes.addFilter(HY_PKG, HY_EQ, resultPset);
+            obsoletes.available();
+
+            Query possibly_obsoleted(sack, ExcludeFlags::IGNORE_EXCLUDES);
+            possibly_obsoleted.addFilter(HY_PKG, HY_EQ, resultPset);
+            possibly_obsoleted.addFilter(HY_PKG_UPGRADES, HY_EQ, 1);
+            possibly_obsoleted.queryUnion(installed);
+            possibly_obsoleted.apply();
+
+            obsoletes.addFilter(HY_PKG_OBSOLETES, HY_EQ, possibly_obsoleted.runSet());
+            obsoletes.apply();
+            Id obsoleted_id = -1;
+            // Add to candidates resultPset pkgs that obsolete some installed (or to be installed in this transaction) pkg
+            while ((obsoleted_id = obsoletes.pImpl->result->next(obsoleted_id)) != -1) {
+                Solvable * s = pool_id2solvable(pool, obsoleted_id);
+                candidates.push_back(s);
+            }
+
+            Id id = -1;
+            // Add to candidates resultPset pkgs that match name and arch with some already installed pkg
+            while ((id = resultPset->next(id)) != -1) {
+                Solvable * s = pool_id2solvable(pool, id);
+                auto low = std::lower_bound(installed_solvables.begin(), installed_solvables.end(), s, NameArchSolvableComparator);
+                if (low != installed_solvables.end() && s->name == (*low)->name && s->arch == (*low)->arch) {
+                    candidates.push_back(s);
+                }
+            }
+
+            // Apply security filters only to packages with lower priority - to unify behaviour upgrade
+            // and upgrade-minimal
+            std::sort(candidates.begin(), candidates.end(), NameArchPrioritySolvableKey);
+            std::vector<Solvable *> priority_candidates;
+            Id name = 0;
+            Id arch = 0;
+            int priority = 0;
+
+            for (auto * candidate: candidates) {
+                if (candidate->repo == pool->installed) {
+                    priority_candidates.push_back(candidate);
+                } else if (name != candidate->name || arch != candidate->arch) {
+                    name = candidate->name;
+                    arch = candidate->arch;
+                    priority = candidate->repo->priority;
+                    priority_candidates.push_back(candidate);
+                } else if (priority == candidate->repo->priority) {
+                    priority_candidates.push_back(candidate);
+                }
+            }
+            std::swap(candidates, priority_candidates);
         } else {
-            auto low = std::lower_bound(pkgs.begin(), pkgs.end(), *s, advisoryPkgCompareSolvableNameArch);
-            while (low != pkgs.end() && low->getName() == s->name && low->getArch() == s->arch) {
-                int cmp = pool_evrcmp(pool, s->evr, low->getEVR(), EVRCMP_COMPARE);
-                if ((cmp > 0 && cmp_type & HY_GT) ||
-                    (cmp < 0 && cmp_type & HY_LT) ||
-                    (cmp == 0 && cmp_type & HY_EQ)) {
+            Id id = -1;
+            while ((id = resultPset->next(id)) != -1) {
+                candidates.push_back(pool_id2solvable(pool, id));
+            }
+        }
+
+        NameArchEVRComparator cmp_key(pool);
+        std::sort(candidates.begin(), candidates.end(), cmp_key);
+        for (auto & advisoryPkg : pkgs) {
+            if (cmp_type & HY_UPGRADE) {
+                // skip advisory pkgs that have lower evr than installed version - important for upgrade logic
+                auto low = std::lower_bound(installed_solvables.begin(), installed_solvables.end(), advisoryPkg, SolvableCompareAdvisoryPkgNameArch);
+                if (low != installed_solvables.end() && advisoryPkg.getName() == (*low)->name && advisoryPkg.getArch() == (*low)->arch) {
+                    // Skip all advisory packages that has same or lover ever than installed
+                    if (pool_evrcmp(pool, (*low)->evr, advisoryPkg.getEVR(), EVRCMP_COMPARE) >= 0) {
+                        continue;
+                    }
+                }
+            }
+            auto low = std::lower_bound(candidates.begin(), candidates.end(), advisoryPkg, cmp_key);
+            if (low != candidates.end() && advisoryPkg.getName() == (*low)->name && advisoryPkg.getArch() == (*low)->arch) {
+                MAPSET(m, pool_solvable2id(pool, (*low)));
+                if (cmp_type & HY_GT) {
+                    ++low;
+                    while (low != candidates.end() && advisoryPkg.getName() == (*low)->name && advisoryPkg.getArch() == (*low)->arch) {
+                        MAPSET(m, pool_solvable2id(pool, (*low)));
+                        ++low;
+                    }
+                }
+            }
+        }
+    } else {
+        // convert nevras (from DnfAdvisoryPkg) to pool ids
+        Id id = -1;
+        while (true) {
+            if (pkgs.size() == 0)
+                break;
+            id = resultPset->next(id);
+            if (id == -1)
+                break;
+            Solvable* s = pool_id2solvable(pool, id);
+            if (cmp_type == HY_EQ) {
+                auto low = std::lower_bound(pkgs.begin(), pkgs.end(), *s, advisoryPkgCompareSolvable);
+                if (low != pkgs.end() && low->nevraEQ(s)) {
                     MAPSET(m, id);
-                    break;
+                }
+            } else {
+                auto low = std::lower_bound(pkgs.begin(), pkgs.end(), *s, advisoryPkgCompareSolvableNameArch);
+                while (low != pkgs.end() && low->getName() == s->name && low->getArch() == s->arch) {
+                    int cmp = pool_evrcmp(pool, s->evr, low->getEVR(), EVRCMP_COMPARE);
+                    if ((cmp > 0 && cmp_type & HY_GT) ||
+                        (cmp < 0 && cmp_type & HY_LT) ||
+                        (cmp == 0 && cmp_type & HY_EQ)) {
+                        MAPSET(m, id);
+                        break;
+                    }
+                    ++low;
                 }
-                ++low;
             }
         }
     }
@@ -2434,9 +2585,7 @@ Query::getAdvisoryPkgs(int cmpType, std::vector<AdvisoryPkg> & advisoryPkgs)
     dataiterator_prepend_keyname(&di, UPDATE_COLLECTION);
     while (dataiterator_step(&di)) {
         Advisory advisory(sack, di.solvid);
-        if (isAdvisoryApplicable(advisory, sack)) {
-            advisory.getPackages(pkgs);
-        }
+        advisory.getApplicablePackages(pkgs);
         dataiterator_skip_solvable(&di);
     }
     dataiterator_free(&di);
