@@ -161,7 +161,7 @@ dnf_lock_threads_func(void)
     guint lock_id;
     DnfLock *lock;
 
-    /* take in master thread */
+    /* take in parent thread */
     lock = dnf_lock_new();
     dnf_lock_set_lock_dir(lock, "/tmp");
     lock_id = dnf_lock_take(lock,
@@ -171,7 +171,7 @@ dnf_lock_threads_func(void)
     g_assert_no_error(error);
     g_assert_cmpint(lock_id, >, 0);
 
-    /* attempt to take in slave thread(should fail) */
+    /* attempt to take in child thread(should fail) */
     one = g_thread_new("dnf-lock-one",
                 dnf_self_test_lock_thread_one,
                 lock);
@@ -199,6 +199,26 @@ ch_test_repo_func(void)
     g_object_unref(context);
 }
 
+static void
+dnf_repo_setup_with_empty_keyfile(void)
+{
+    DnfContext *context = dnf_context_new();
+    dnf_context_set_release_ver(context, "33");
+    DnfRepo *repo = dnf_repo_new(context);
+    GKeyFile *empty_key_file = g_key_file_new();
+
+    dnf_repo_set_keyfile(repo, empty_key_file);
+
+    GError *error = NULL;
+    // Empty key file is not an errror, there is just no configuration to be loaded.
+    g_assert(dnf_repo_setup(repo, &error));
+    g_assert_no_error(error);
+
+    g_object_unref(repo);
+    g_object_unref(context);
+    g_key_file_free(empty_key_file);
+}
+
 static guint _allow_cancel_updates = 0;
 static guint _action_updates = 0;
 static guint _package_progress_updates = 0;
@@ -885,6 +905,7 @@ dnf_context_func(void)
     g_assert_cmpstr(dnf_context_get_solv_dir(ctx), ==, "/tmp/hawkey");
     g_assert(dnf_context_get_check_disk_space(ctx));
     g_assert(dnf_context_get_check_transaction(ctx));
+    dnf_context_set_keep_cache(ctx, FALSE);
     g_assert(!dnf_context_get_keep_cache(ctx));
 
     dnf_context_set_cache_dir(ctx, "/var");
@@ -1191,12 +1212,12 @@ dnf_context_cache_clean_check_func(void)
 int
 main(int argc, char **argv)
 {
-    g_setenv("G_MESSAGES_DEBUG", "all", FALSE);
+    g_assert(g_setenv("G_MESSAGES_DEBUG", "all", FALSE));
     /* avoid gvfs (http://bugzilla.gnome.org/show_bug.cgi?id=526454) */
     /* Also because we do valgrind testing and there are vast array of
      * "leaks" when we load gio vfs modules.
      */
-    g_setenv ("GIO_USE_VFS", "local", TRUE);
+    g_assert(g_setenv ("GIO_USE_VFS", "local", TRUE));
 
     g_test_init(&argc, &argv, NULL);
 
@@ -1204,6 +1225,9 @@ main(int argc, char **argv)
     g_log_set_fatal_mask(NULL, G_LOG_LEVEL_ERROR | G_LOG_LEVEL_CRITICAL);
     g_log_set_always_fatal (G_LOG_FATAL_MASK);
 
+    /* Sets a variable to replace in repository configurations. */
+    g_assert(g_setenv("DNF_VAR_testdatadir", TESTDATADIR, TRUE));
+
     /* tests go here */
     g_test_add_func("/libdnf/repo_loader{gpg-asc}", dnf_repo_loader_gpg_asc_func);
     g_test_add_func("/libdnf/repo_loader{gpg-wrong-asc}", dnf_repo_loader_gpg_wrong_asc_func);
@@ -1216,6 +1240,7 @@ main(int argc, char **argv)
     g_test_add_func("/libdnf/lock", dnf_lock_func);
     g_test_add_func("/libdnf/lock[threads]", dnf_lock_threads_func);
     g_test_add_func("/libdnf/repo", ch_test_repo_func);
+    g_test_add_func("/libdnf/repo_empty_keyfile", dnf_repo_setup_with_empty_keyfile);
     g_test_add_func("/libdnf/state", dnf_state_func);
     g_test_add_func("/libdnf/state[child]", dnf_state_child_func);
     g_test_add_func("/libdnf/state[parent-1-step]", dnf_state_parent_one_step_proxy_func);
